from typing import overload
import abc
import typing

import Calculators.DataType
import Calculators.Margins
import System
import System.Collections.Concurrent
import System.Collections.Generic


class IFTXMarginModel(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def CalculateMarginFraction(self, holdings: System.Collections.Generic.IList[Calculators.DataType.Holding], isInitial: bool) -> float:
        ...

    def GetAvailableCollateral(self, holdings: System.Collections.Generic.IList[Calculators.DataType.Holding]) -> float:
        ...

    def GetCollateral(self, holdings: System.Collections.Generic.IList[Calculators.DataType.Holding], isInitial: bool) -> float:
        ...

    def IsAtRiskForLiquidation(self, holdings: System.Collections.Generic.IList[Calculators.DataType.Holding], riskControlMultiplier: float) -> bool:
        ...

    def TryUpdateCollateralParameters(self, coin: str, totalWeight: float) -> None:
        ...


class FTXMarginModel(System.Object, Calculators.Margins.IFTXMarginModel):
    """This class has no documentation."""

    @property
    def _quoteCurrency(self) -> str:
        """This field is protected."""
        ...

    _collateralParametersMap: System.Collections.Concurrent.ConcurrentDictionary[str, Calculators.DataType.FTXCollateralParameters]
    """This field is protected."""

    @property
    def _logger(self) -> typing.Any:
        """This field is protected."""
        ...

    @_logger.setter
    def _logger(self, value: typing.Any):
        """This field is protected."""
        ...

    def __init__(self, location: str = ..., quoteCurrency: str = "USD") -> None:
        ...

    def CalculateInitialMargin(self, holding: Calculators.DataType.Holding) -> float:
        """This method is protected."""
        ...

    def CalculateMarginFraction(self, holdings: System.Collections.Generic.IList[Calculators.DataType.Holding], isInitial: bool) -> float:
        ...

    def CalculateNotional(self, holding: Calculators.DataType.Holding) -> float:
        """This method is protected."""
        ...

    def GetAvailableCollateral(self, holdings: System.Collections.Generic.IList[Calculators.DataType.Holding]) -> float:
        """
        collateral used is equal to:
            For PRESIDENT: initialMarginRequirement * openSize * (risk price)
            For MOVE: initialMarginRequirement * openSize * (index price)
            Otherwise: initialMarginRequirement * openSize * (mark price)
        """
        ...

    def GetCollateral(self, holdings: System.Collections.Generic.IList[Calculators.DataType.Holding], isInitial: bool) -> float:
        ...

    @overload
    def IsAtRiskForLiquidation(self, holdings: System.Collections.Generic.IList[Calculators.DataType.Holding], riskControlMultiplier: float = 1) -> bool:
        """
        FTX will trade your non-USD collateral into USD if your USD balance is negative and any of the following hold:
            You are close to liquidation: your account's margin fraction is less than (20bps + maintenance margin fraction requirement)
            Your negative USD balance is large: over $30,000 in magnitude
            Your negative USD balance is large when compared to overall collateral: its magnitude is over 4 times larger than your net account collateral
        From: https://help.ftx.com/hc/en-us/articles/360031149632-Non-USD-Collateral
        
        :param riskControlMultiplier: a multiplier to adjust USD remaining requirement, should be between 0-1 to strengthen the rules
        """
        ...

    @overload
    def IsAtRiskForLiquidation(self, marginFraction: float, weightedAverageMaintenanceMarginRequirement: float, usdPosition: float, netCollateral: float) -> bool:
        ...

    def TryUpdateCollateralParameters(self, coin: str, totalWeight: float) -> None:
        ...


class FTXMarginModelFactory(System.Object):
    """This class has no documentation."""

    @staticmethod
    def Init(location: str) -> None:
        ...

    @staticmethod
    def Resolve(enableSpotMargin: bool) -> Calculators.Margins.IFTXMarginModel:
        ...


class FTXMarginModelWithSpotMargin(Calculators.Margins.FTXMarginModel):
    """This class has no documentation."""

    def __init__(self, location: str = ..., quoteCurrency: str = "USD") -> None:
        ...

    def CalculateInitialMargin(self, holding: Calculators.DataType.Holding) -> float:
        """This method is protected."""
        ...

    def CalculateNotional(self, holding: Calculators.DataType.Holding) -> float:
        """This method is protected."""
        ...

    def GetInitialMarginRequirementForCrypto(self, holding: Calculators.DataType.Holding) -> float:
        ...

    def GetMaintenanceMarginRequirementForCrypto(self, holding: Calculators.DataType.Holding) -> float:
        ...

    def IsAtRiskForLiquidation(self, holdings: System.Collections.Generic.IList[Calculators.DataType.Holding], riskControlMultiplier: float = 1) -> bool:
        """
        The auto-close margin--the point at which you are not just liquidated but in fact closed down off-exchange--is 50% of the maintenance margin requirement
        From: https://help.ftx.com/hc/en-us/articles/360053007671
        for safety, we send liquidation warning once reach mantenance margin
        """
        ...





from typing import overload
import datetime
import typing

import FtxApi.Rest.Models
import System
import System.Collections.Generic

FtxApi_Rest_Models_FtxResult_T = typing.TypeVar("FtxApi_Rest_Models_FtxResult_T")


class Position(System.Object):
    """This class has no documentation."""

    @property
    def CollateralUsed(self) -> typing.Optional[float]:
        ...

    @CollateralUsed.setter
    def CollateralUsed(self, value: typing.Optional[float]):
        ...

    @property
    def Cost(self) -> typing.Optional[float]:
        ...

    @Cost.setter
    def Cost(self, value: typing.Optional[float]):
        ...

    @property
    def EntryPrice(self) -> typing.Optional[float]:
        ...

    @EntryPrice.setter
    def EntryPrice(self, value: typing.Optional[float]):
        ...

    @property
    def EstimatedLiquidationPrice(self) -> typing.Optional[float]:
        ...

    @EstimatedLiquidationPrice.setter
    def EstimatedLiquidationPrice(self, value: typing.Optional[float]):
        ...

    @property
    def Future(self) -> str:
        ...

    @Future.setter
    def Future(self, value: str):
        ...

    @property
    def InitialMarginRequirement(self) -> typing.Optional[float]:
        ...

    @InitialMarginRequirement.setter
    def InitialMarginRequirement(self, value: typing.Optional[float]):
        ...

    @property
    def LongOrderSize(self) -> typing.Optional[float]:
        ...

    @LongOrderSize.setter
    def LongOrderSize(self, value: typing.Optional[float]):
        ...

    @property
    def MaintenanceMarginRequirement(self) -> typing.Optional[float]:
        ...

    @MaintenanceMarginRequirement.setter
    def MaintenanceMarginRequirement(self, value: typing.Optional[float]):
        ...

    @property
    def NetSize(self) -> typing.Optional[float]:
        ...

    @NetSize.setter
    def NetSize(self, value: typing.Optional[float]):
        ...

    @property
    def OpenSize(self) -> typing.Optional[float]:
        ...

    @OpenSize.setter
    def OpenSize(self, value: typing.Optional[float]):
        ...

    @property
    def RealizedPnl(self) -> typing.Optional[float]:
        ...

    @RealizedPnl.setter
    def RealizedPnl(self, value: typing.Optional[float]):
        ...

    @property
    def ShortOrderSize(self) -> typing.Optional[float]:
        ...

    @ShortOrderSize.setter
    def ShortOrderSize(self, value: typing.Optional[float]):
        ...

    @property
    def Side(self) -> str:
        ...

    @Side.setter
    def Side(self, value: str):
        ...

    @property
    def Size(self) -> typing.Optional[float]:
        ...

    @Size.setter
    def Size(self, value: typing.Optional[float]):
        ...

    @property
    def UnrealizedPnl(self) -> typing.Optional[float]:
        ...

    @UnrealizedPnl.setter
    def UnrealizedPnl(self, value: typing.Optional[float]):
        ...


class AccountInfo(System.Object):
    """This class has no documentation."""

    @property
    def BackstopProvider(self) -> bool:
        ...

    @BackstopProvider.setter
    def BackstopProvider(self, value: bool):
        ...

    @property
    def Collateral(self) -> float:
        ...

    @Collateral.setter
    def Collateral(self, value: float):
        ...

    @property
    def FreeCollateral(self) -> float:
        ...

    @FreeCollateral.setter
    def FreeCollateral(self, value: float):
        ...

    @property
    def InitialMarginRequirement(self) -> float:
        ...

    @InitialMarginRequirement.setter
    def InitialMarginRequirement(self, value: float):
        ...

    @property
    def Liquidating(self) -> bool:
        ...

    @Liquidating.setter
    def Liquidating(self, value: bool):
        ...

    @property
    def MaintenanceMarginRequirement(self) -> float:
        ...

    @MaintenanceMarginRequirement.setter
    def MaintenanceMarginRequirement(self, value: float):
        ...

    @property
    def MakerFee(self) -> float:
        ...

    @MakerFee.setter
    def MakerFee(self, value: float):
        ...

    @property
    def MarginFraction(self) -> typing.Optional[float]:
        ...

    @MarginFraction.setter
    def MarginFraction(self, value: typing.Optional[float]):
        ...

    @property
    def OpenMarginFraction(self) -> typing.Optional[float]:
        ...

    @OpenMarginFraction.setter
    def OpenMarginFraction(self, value: typing.Optional[float]):
        ...

    @property
    def TakerFee(self) -> float:
        ...

    @TakerFee.setter
    def TakerFee(self, value: float):
        ...

    @property
    def TotalAccountValue(self) -> float:
        ...

    @TotalAccountValue.setter
    def TotalAccountValue(self, value: float):
        ...

    @property
    def TotalPositionSize(self) -> float:
        ...

    @TotalPositionSize.setter
    def TotalPositionSize(self, value: float):
        ...

    @property
    def Username(self) -> str:
        ...

    @Username.setter
    def Username(self, value: str):
        ...

    @property
    def Leverage(self) -> float:
        ...

    @Leverage.setter
    def Leverage(self, value: float):
        ...

    @property
    def Positions(self) -> System.Collections.Generic.List[FtxApi.Rest.Models.Position]:
        ...

    @Positions.setter
    def Positions(self, value: System.Collections.Generic.List[FtxApi.Rest.Models.Position]):
        ...


class AccountLeverage(System.Object):
    """This class has no documentation."""

    @property
    def Leverage(self) -> int:
        ...

    @Leverage.setter
    def Leverage(self, value: int):
        ...


class Balance(System.Object):
    """This class has no documentation."""

    @property
    def Coin(self) -> str:
        ...

    @Coin.setter
    def Coin(self, value: str):
        ...

    @property
    def Free(self) -> float:
        ...

    @Free.setter
    def Free(self, value: float):
        ...

    @property
    def Total(self) -> float:
        ...

    @Total.setter
    def Total(self, value: float):
        ...


class Candle(System.Object):
    """This class has no documentation."""

    @property
    def Close(self) -> float:
        ...

    @Close.setter
    def Close(self, value: float):
        ...

    @property
    def High(self) -> float:
        ...

    @High.setter
    def High(self, value: float):
        ...

    @property
    def Low(self) -> float:
        ...

    @Low.setter
    def Low(self, value: float):
        ...

    @property
    def Open(self) -> float:
        ...

    @Open.setter
    def Open(self, value: float):
        ...

    @property
    def StartTime(self) -> datetime.datetime:
        ...

    @StartTime.setter
    def StartTime(self, value: datetime.datetime):
        ...

    @property
    def Volume(self) -> float:
        ...

    @Volume.setter
    def Volume(self, value: float):
        ...


class Coin(System.Object):
    """This class has no documentation."""

    @property
    def CanDeposit(self) -> bool:
        ...

    @CanDeposit.setter
    def CanDeposit(self, value: bool):
        ...

    @property
    def CanWithdraw(self) -> bool:
        ...

    @CanWithdraw.setter
    def CanWithdraw(self, value: bool):
        ...

    @property
    def HasTag(self) -> bool:
        ...

    @HasTag.setter
    def HasTag(self, value: bool):
        ...

    @property
    def Id(self) -> str:
        ...

    @Id.setter
    def Id(self, value: str):
        ...

    @property
    def Name(self) -> str:
        ...

    @Name.setter
    def Name(self, value: str):
        ...


class DepositAddress(System.Object):
    """This class has no documentation."""

    @property
    def Address(self) -> str:
        ...

    @Address.setter
    def Address(self, value: str):
        ...

    @property
    def Tag(self) -> str:
        ...

    @Tag.setter
    def Tag(self, value: str):
        ...


class DepositHistory(System.Object):
    """This class has no documentation."""

    @property
    def Coin(self) -> str:
        ...

    @Coin.setter
    def Coin(self, value: str):
        ...

    @property
    def Confirmations(self) -> float:
        ...

    @Confirmations.setter
    def Confirmations(self, value: float):
        ...

    @property
    def ConfirmedTime(self) -> datetime.datetime:
        ...

    @ConfirmedTime.setter
    def ConfirmedTime(self, value: datetime.datetime):
        ...

    @property
    def Fee(self) -> float:
        ...

    @Fee.setter
    def Fee(self, value: float):
        ...

    @property
    def Id(self) -> float:
        ...

    @Id.setter
    def Id(self, value: float):
        ...

    @property
    def SentTime(self) -> datetime.datetime:
        ...

    @SentTime.setter
    def SentTime(self, value: datetime.datetime):
        ...

    @property
    def Size(self) -> float:
        ...

    @Size.setter
    def Size(self, value: float):
        ...

    @property
    def Status(self) -> str:
        ...

    @Status.setter
    def Status(self, value: str):
        ...

    @property
    def Time(self) -> datetime.datetime:
        ...

    @Time.setter
    def Time(self, value: datetime.datetime):
        ...

    @property
    def TxId(self) -> str:
        ...

    @TxId.setter
    def TxId(self, value: str):
        ...

    @property
    def Notes(self) -> str:
        ...

    @Notes.setter
    def Notes(self, value: str):
        ...


class Fill(System.Object):
    """This class has no documentation."""

    @property
    def Fee(self) -> float:
        ...

    @Fee.setter
    def Fee(self, value: float):
        ...

    @property
    def FeeCurrency(self) -> str:
        ...

    @FeeCurrency.setter
    def FeeCurrency(self, value: str):
        ...

    @property
    def FeeRate(self) -> typing.Optional[float]:
        ...

    @FeeRate.setter
    def FeeRate(self, value: typing.Optional[float]):
        ...

    @property
    def Future(self) -> str:
        ...

    @Future.setter
    def Future(self, value: str):
        ...

    @property
    def Id(self) -> float:
        ...

    @Id.setter
    def Id(self, value: float):
        ...

    @property
    def Liquidity(self) -> str:
        ...

    @Liquidity.setter
    def Liquidity(self, value: str):
        ...

    @property
    def Market(self) -> str:
        ...

    @Market.setter
    def Market(self, value: str):
        ...

    @property
    def BaseCurrency(self) -> str:
        ...

    @BaseCurrency.setter
    def BaseCurrency(self, value: str):
        ...

    @property
    def QuoteCurrency(self) -> str:
        ...

    @QuoteCurrency.setter
    def QuoteCurrency(self, value: str):
        ...

    @property
    def OrderId(self) -> int:
        ...

    @OrderId.setter
    def OrderId(self, value: int):
        ...

    @property
    def TradeId(self) -> int:
        ...

    @TradeId.setter
    def TradeId(self, value: int):
        ...

    @property
    def Price(self) -> float:
        ...

    @Price.setter
    def Price(self, value: float):
        ...

    @property
    def Side(self) -> str:
        ...

    @Side.setter
    def Side(self, value: str):
        ...

    @property
    def Size(self) -> float:
        ...

    @Size.setter
    def Size(self, value: float):
        ...

    @property
    def Time(self) -> datetime.datetime:
        ...

    @Time.setter
    def Time(self, value: datetime.datetime):
        ...

    @property
    def Type(self) -> str:
        ...

    @Type.setter
    def Type(self, value: str):
        ...


class FtxOption(System.Object):
    """This class has no documentation."""

    @property
    def Underlying(self) -> str:
        ...

    @Underlying.setter
    def Underlying(self, value: str):
        ...

    @property
    def Type(self) -> str:
        ...

    @Type.setter
    def Type(self, value: str):
        ...

    @property
    def Strike(self) -> float:
        ...

    @Strike.setter
    def Strike(self, value: float):
        ...

    @property
    def Expiry(self) -> datetime.datetime:
        ...

    @Expiry.setter
    def Expiry(self, value: datetime.datetime):
        ...

    def Parse(self, symbol: str) -> None:
        ...

    @staticmethod
    def ParseExpiry(expiry: str) -> datetime.datetime:
        ...

    def ToString(self) -> str:
        ...


class FtxResult(typing.Generic[FtxApi_Rest_Models_FtxResult_T], System.Object):
    """This class has no documentation."""

    @property
    def Success(self) -> bool:
        ...

    @Success.setter
    def Success(self, value: bool):
        ...

    @property
    def Result(self) -> FtxApi_Rest_Models_FtxResult_T:
        ...

    @Result.setter
    def Result(self, value: FtxApi_Rest_Models_FtxResult_T):
        ...

    @property
    def Error(self) -> str:
        ...

    @Error.setter
    def Error(self, value: str):
        ...

    @property
    def Request(self) -> str:
        ...

    @Request.setter
    def Request(self, value: str):
        ...


class FundingPayment(System.Object):
    """This class has no documentation."""

    @property
    def Future(self) -> str:
        ...

    @Future.setter
    def Future(self, value: str):
        ...

    @property
    def Id(self) -> float:
        ...

    @Id.setter
    def Id(self, value: float):
        ...

    @property
    def Payment(self) -> float:
        ...

    @Payment.setter
    def Payment(self, value: float):
        ...

    @property
    def Time(self) -> datetime.datetime:
        ...

    @Time.setter
    def Time(self, value: datetime.datetime):
        ...


class FundingRate(System.Object):
    """This class has no documentation."""

    @property
    def Future(self) -> str:
        ...

    @Future.setter
    def Future(self, value: str):
        ...

    @property
    def Rate(self) -> float:
        ...

    @Rate.setter
    def Rate(self, value: float):
        ...

    @property
    def Time(self) -> datetime.datetime:
        ...

    @Time.setter
    def Time(self, value: datetime.datetime):
        ...


class Future(System.Object):
    """This class has no documentation."""

    @property
    def Ask(self) -> typing.Optional[float]:
        """best ask on the orderbook"""
        ...

    @Ask.setter
    def Ask(self, value: typing.Optional[float]):
        """best ask on the orderbook"""
        ...

    @property
    def Bid(self) -> typing.Optional[float]:
        """best bid on the orderbook"""
        ...

    @Bid.setter
    def Bid(self, value: typing.Optional[float]):
        """best bid on the orderbook"""
        ...

    @property
    def Change1H(self) -> float:
        """price change in the last hour"""
        ...

    @Change1H.setter
    def Change1H(self, value: float):
        """price change in the last hour"""
        ...

    @property
    def Change24H(self) -> float:
        """price change in the last 24 hours"""
        ...

    @Change24H.setter
    def Change24H(self, value: float):
        """price change in the last 24 hours"""
        ...

    @property
    def ChangeBod(self) -> float:
        """price change since midnight UTC (beginning of day)"""
        ...

    @ChangeBod.setter
    def ChangeBod(self, value: float):
        """price change since midnight UTC (beginning of day)"""
        ...

    @property
    def VolumeUsd24H(self) -> float:
        """USD volume in the last 24 hours"""
        ...

    @VolumeUsd24H.setter
    def VolumeUsd24H(self, value: float):
        """USD volume in the last 24 hours"""
        ...

    @property
    def Volume(self) -> float:
        """quantity traded in the last 24 hours"""
        ...

    @Volume.setter
    def Volume(self, value: float):
        """quantity traded in the last 24 hours"""
        ...

    @property
    def Description(self) -> str:
        ...

    @Description.setter
    def Description(self, value: str):
        ...

    @property
    def Enabled(self) -> bool:
        ...

    @Enabled.setter
    def Enabled(self, value: bool):
        ...

    @property
    def Expired(self) -> bool:
        ...

    @Expired.setter
    def Expired(self, value: bool):
        ...

    @property
    def Expiry(self) -> typing.Optional[datetime.datetime]:
        ...

    @Expiry.setter
    def Expiry(self, value: typing.Optional[datetime.datetime]):
        ...

    @property
    def Index(self) -> float:
        """average of the Market Prices for the constituent markets in the index"""
        ...

    @Index.setter
    def Index(self, value: float):
        """average of the Market Prices for the constituent markets in the index"""
        ...

    @property
    def ImfFactor(self) -> float:
        ...

    @ImfFactor.setter
    def ImfFactor(self, value: float):
        ...

    @property
    def Last(self) -> typing.Optional[float]:
        """last price the future traded at"""
        ...

    @Last.setter
    def Last(self, value: typing.Optional[float]):
        """last price the future traded at"""
        ...

    @property
    def LowerBound(self) -> float:
        """the lowest price the future can trade at"""
        ...

    @LowerBound.setter
    def LowerBound(self, value: float):
        """the lowest price the future can trade at"""
        ...

    @property
    def Mark(self) -> float:
        """mark price of the future"""
        ...

    @Mark.setter
    def Mark(self, value: float):
        """mark price of the future"""
        ...

    @property
    def Name(self) -> str:
        ...

    @Name.setter
    def Name(self, value: str):
        ...

    @property
    def Perpetual(self) -> bool:
        """whether or not this is a perpetual contract"""
        ...

    @Perpetual.setter
    def Perpetual(self, value: bool):
        """whether or not this is a perpetual contract"""
        ...

    @property
    def PositionLimitWeight(self) -> float:
        ...

    @PositionLimitWeight.setter
    def PositionLimitWeight(self, value: float):
        ...

    @property
    def PostOnly(self) -> bool:
        ...

    @PostOnly.setter
    def PostOnly(self, value: bool):
        ...

    @property
    def PriceIncrement(self) -> float:
        ...

    @PriceIncrement.setter
    def PriceIncrement(self, value: float):
        ...

    @property
    def SizeIncrement(self) -> float:
        ...

    @SizeIncrement.setter
    def SizeIncrement(self, value: float):
        ...

    @property
    def Underlying(self) -> str:
        ...

    @Underlying.setter
    def Underlying(self, value: str):
        ...

    @property
    def UpperBound(self) -> float:
        """the highest price the future can trade at"""
        ...

    @UpperBound.setter
    def UpperBound(self, value: float):
        """the highest price the future can trade at"""
        ...

    @property
    def Type(self) -> str:
        """One of future, perpetual, or move"""
        ...

    @Type.setter
    def Type(self, value: str):
        """One of future, perpetual, or move"""
        ...

    @property
    def MoveStart(self) -> str:
        ...

    @MoveStart.setter
    def MoveStart(self, value: str):
        ...


class FutureStats(System.Object):
    """This class has no documentation."""

    @property
    def Volume(self) -> float:
        ...

    @Volume.setter
    def Volume(self, value: float):
        ...

    @property
    def NextFundingRate(self) -> float:
        ...

    @NextFundingRate.setter
    def NextFundingRate(self, value: float):
        ...

    @property
    def NextFundingTime(self) -> str:
        ...

    @NextFundingTime.setter
    def NextFundingTime(self, value: str):
        ...

    @property
    def ExpirationPrice(self) -> float:
        ...

    @ExpirationPrice.setter
    def ExpirationPrice(self, value: float):
        ...

    @property
    def PredictedExpirationPrice(self) -> float:
        ...

    @PredictedExpirationPrice.setter
    def PredictedExpirationPrice(self, value: float):
        ...

    @property
    def OpenInterest(self) -> float:
        ...

    @OpenInterest.setter
    def OpenInterest(self, value: float):
        ...

    @property
    def StrikePrice(self) -> float:
        ...

    @StrikePrice.setter
    def StrikePrice(self, value: float):
        ...


class OptionFill(System.Object):
    """This class has no documentation."""

    @property
    def Id(self) -> int:
        ...

    @Id.setter
    def Id(self, value: int):
        ...

    @property
    def Size(self) -> float:
        ...

    @Size.setter
    def Size(self, value: float):
        ...

    @property
    def Price(self) -> float:
        ...

    @Price.setter
    def Price(self, value: float):
        ...

    @property
    def Option(self) -> FtxApi.Rest.Models.FtxOption:
        ...

    @Option.setter
    def Option(self, value: FtxApi.Rest.Models.FtxOption):
        ...

    @property
    def Time(self) -> datetime.datetime:
        ...

    @Time.setter
    def Time(self, value: datetime.datetime):
        ...

    @property
    def Liquidity(self) -> str:
        ...

    @Liquidity.setter
    def Liquidity(self, value: str):
        ...

    @property
    def Fee(self) -> float:
        ...

    @Fee.setter
    def Fee(self, value: float):
        ...

    @property
    def FeeRate(self) -> float:
        ...

    @FeeRate.setter
    def FeeRate(self, value: float):
        ...

    @property
    def Side(self) -> str:
        ...

    @Side.setter
    def Side(self, value: str):
        ...


class OptionPosition(System.Object):
    """This class has no documentation."""

    @property
    def NetSize(self) -> float:
        ...

    @NetSize.setter
    def NetSize(self, value: float):
        ...

    @property
    def EntryPrice(self) -> float:
        ...

    @EntryPrice.setter
    def EntryPrice(self, value: float):
        ...

    @property
    def Size(self) -> float:
        ...

    @Size.setter
    def Size(self, value: float):
        ...

    @property
    def Option(self) -> FtxApi.Rest.Models.FtxOption:
        ...

    @Option.setter
    def Option(self, value: FtxApi.Rest.Models.FtxOption):
        ...

    @property
    def Side(self) -> str:
        ...

    @Side.setter
    def Side(self, value: str):
        ...

    @property
    def PessimisticValuation(self) -> typing.Optional[float]:
        ...

    @PessimisticValuation.setter
    def PessimisticValuation(self, value: typing.Optional[float]):
        ...

    @property
    def PessimisticIndexPrice(self) -> typing.Optional[float]:
        ...

    @PessimisticIndexPrice.setter
    def PessimisticIndexPrice(self, value: typing.Optional[float]):
        ...

    @property
    def PessimisticVol(self) -> typing.Optional[float]:
        ...

    @PessimisticVol.setter
    def PessimisticVol(self, value: typing.Optional[float]):
        ...


class OptionQuote(System.Object):
    """This class has no documentation."""

    @property
    def Collateral(self) -> float:
        ...

    @Collateral.setter
    def Collateral(self, value: float):
        ...

    @property
    def Id(self) -> int:
        ...

    @Id.setter
    def Id(self, value: int):
        ...

    @property
    def Option(self) -> FtxApi.Rest.Models.FtxOption:
        ...

    @Option.setter
    def Option(self, value: FtxApi.Rest.Models.FtxOption):
        ...

    @property
    def Price(self) -> float:
        ...

    @Price.setter
    def Price(self, value: float):
        ...

    @property
    def QuoteExpiry(self) -> typing.Optional[datetime.datetime]:
        ...

    @QuoteExpiry.setter
    def QuoteExpiry(self, value: typing.Optional[datetime.datetime]):
        ...

    @property
    def QuoterSide(self) -> str:
        ...

    @QuoterSide.setter
    def QuoterSide(self, value: str):
        ...

    @property
    def RequestId(self) -> typing.Optional[int]:
        ...

    @RequestId.setter
    def RequestId(self, value: typing.Optional[int]):
        ...

    @property
    def RequestSide(self) -> str:
        ...

    @RequestSide.setter
    def RequestSide(self, value: str):
        ...

    @property
    def Size(self) -> typing.Optional[float]:
        ...

    @Size.setter
    def Size(self, value: typing.Optional[float]):
        ...

    @property
    def Status(self) -> str:
        ...

    @Status.setter
    def Status(self, value: str):
        ...

    @property
    def Time(self) -> datetime.datetime:
        ...

    @Time.setter
    def Time(self, value: datetime.datetime):
        ...

    def ToString(self) -> str:
        ...


class Order(System.Object):
    """This class has no documentation."""

    @property
    def CreatedAt(self) -> datetime.datetime:
        ...

    @CreatedAt.setter
    def CreatedAt(self, value: datetime.datetime):
        ...

    @property
    def FilledSize(self) -> typing.Optional[float]:
        ...

    @FilledSize.setter
    def FilledSize(self, value: typing.Optional[float]):
        ...

    @property
    def Future(self) -> str:
        ...

    @Future.setter
    def Future(self, value: str):
        ...

    @property
    def Id(self) -> int:
        ...

    @Id.setter
    def Id(self, value: int):
        ...

    @property
    def Market(self) -> str:
        ...

    @Market.setter
    def Market(self, value: str):
        ...

    @property
    def Price(self) -> typing.Optional[float]:
        ...

    @Price.setter
    def Price(self, value: typing.Optional[float]):
        ...

    @property
    def AvgFillPrice(self) -> typing.Optional[float]:
        ...

    @AvgFillPrice.setter
    def AvgFillPrice(self, value: typing.Optional[float]):
        ...

    @property
    def RemainingSize(self) -> typing.Optional[float]:
        ...

    @RemainingSize.setter
    def RemainingSize(self, value: typing.Optional[float]):
        ...

    @property
    def Side(self) -> str:
        ...

    @Side.setter
    def Side(self, value: str):
        ...

    @property
    def Size(self) -> typing.Optional[float]:
        ...

    @Size.setter
    def Size(self, value: typing.Optional[float]):
        ...

    @property
    def Status(self) -> str:
        ...

    @Status.setter
    def Status(self, value: str):
        ...

    @property
    def Type(self) -> str:
        ...

    @Type.setter
    def Type(self, value: str):
        ...

    @property
    def ReduceOnly(self) -> bool:
        ...

    @ReduceOnly.setter
    def ReduceOnly(self, value: bool):
        ...

    @property
    def Ioc(self) -> bool:
        ...

    @Ioc.setter
    def Ioc(self, value: bool):
        ...

    @property
    def PostOnly(self) -> bool:
        ...

    @PostOnly.setter
    def PostOnly(self, value: bool):
        ...

    @property
    def ClientId(self) -> str:
        ...

    @ClientId.setter
    def ClientId(self, value: str):
        ...


class QuoteRequest(System.Object):
    """This class has no documentation."""

    @property
    def Id(self) -> int:
        ...

    @Id.setter
    def Id(self, value: int):
        ...

    @property
    def Option(self) -> FtxApi.Rest.Models.FtxOption:
        ...

    @Option.setter
    def Option(self, value: FtxApi.Rest.Models.FtxOption):
        ...

    @property
    def Side(self) -> str:
        ...

    @Side.setter
    def Side(self, value: str):
        ...

    @property
    def Size(self) -> float:
        ...

    @Size.setter
    def Size(self, value: float):
        ...

    @property
    def Time(self) -> datetime.datetime:
        ...

    @Time.setter
    def Time(self, value: datetime.datetime):
        ...

    @property
    def RequestExpiry(self) -> datetime.datetime:
        ...

    @RequestExpiry.setter
    def RequestExpiry(self, value: datetime.datetime):
        ...

    @property
    def Status(self) -> str:
        ...

    @Status.setter
    def Status(self, value: str):
        ...

    @property
    def HideLimitPrice(self) -> typing.Optional[bool]:
        ...

    @HideLimitPrice.setter
    def HideLimitPrice(self, value: typing.Optional[bool]):
        ...

    @property
    def LimitPrice(self) -> typing.Optional[float]:
        ...

    @LimitPrice.setter
    def LimitPrice(self, value: typing.Optional[float]):
        ...

    @property
    def Quotes(self) -> typing.List[FtxApi.Rest.Models.OptionQuote]:
        ...

    @Quotes.setter
    def Quotes(self, value: typing.List[FtxApi.Rest.Models.OptionQuote]):
        ...

    def ToString(self) -> str:
        ...


class TriggerOrder(System.Object):
    """This class has no documentation."""

    @property
    def CreatedAt(self) -> datetime.datetime:
        ...

    @CreatedAt.setter
    def CreatedAt(self, value: datetime.datetime):
        ...

    @property
    def Future(self) -> str:
        ...

    @Future.setter
    def Future(self, value: str):
        ...

    @property
    def Id(self) -> float:
        ...

    @Id.setter
    def Id(self, value: float):
        ...

    @property
    def Market(self) -> str:
        ...

    @Market.setter
    def Market(self, value: str):
        ...

    @property
    def TriggerPrice(self) -> typing.Optional[float]:
        ...

    @TriggerPrice.setter
    def TriggerPrice(self, value: typing.Optional[float]):
        ...

    @property
    def OrderId(self) -> typing.Optional[float]:
        ...

    @OrderId.setter
    def OrderId(self, value: typing.Optional[float]):
        ...

    @property
    def Side(self) -> str:
        ...

    @Side.setter
    def Side(self, value: str):
        ...

    @property
    def Size(self) -> typing.Optional[float]:
        ...

    @Size.setter
    def Size(self, value: typing.Optional[float]):
        ...

    @property
    def Status(self) -> str:
        ...

    @Status.setter
    def Status(self, value: str):
        ...

    @property
    def Type(self) -> str:
        ...

    @Type.setter
    def Type(self, value: str):
        ...

    @property
    def OrderPrice(self) -> typing.Optional[float]:
        ...

    @OrderPrice.setter
    def OrderPrice(self, value: typing.Optional[float]):
        ...

    @property
    def Error(self) -> str:
        ...

    @Error.setter
    def Error(self, value: str):
        ...

    @property
    def TriggeredAt(self) -> typing.Optional[datetime.datetime]:
        ...

    @TriggeredAt.setter
    def TriggeredAt(self, value: typing.Optional[datetime.datetime]):
        ...

    @property
    def ReduceOnly(self) -> bool:
        ...

    @ReduceOnly.setter
    def ReduceOnly(self, value: bool):
        ...

    @property
    def OrderType(self) -> str:
        ...

    @OrderType.setter
    def OrderType(self, value: str):
        ...

    @property
    def RetryUntilFilled(self) -> bool:
        ...

    @RetryUntilFilled.setter
    def RetryUntilFilled(self, value: bool):
        ...


class WithdrawalHistory(System.Object):
    """This class has no documentation."""

    @property
    def Coin(self) -> str:
        ...

    @Coin.setter
    def Coin(self, value: str):
        ...

    @property
    def Address(self) -> str:
        ...

    @Address.setter
    def Address(self, value: str):
        ...

    @property
    def Tag(self) -> str:
        ...

    @Tag.setter
    def Tag(self, value: str):
        ...

    @property
    def Fee(self) -> float:
        ...

    @Fee.setter
    def Fee(self, value: float):
        ...

    @property
    def Id(self) -> float:
        ...

    @Id.setter
    def Id(self, value: float):
        ...

    @property
    def Size(self) -> float:
        ...

    @Size.setter
    def Size(self, value: float):
        ...

    @property
    def Status(self) -> str:
        ...

    @Status.setter
    def Status(self, value: str):
        ...

    @property
    def Time(self) -> str:
        ...

    @Time.setter
    def Time(self, value: str):
        ...

    @property
    def TxId(self) -> str:
        ...

    @TxId.setter
    def TxId(self, value: str):
        ...





from typing import overload
import datetime
import typing

import MomCrypto.Frontend
import System
import System.Collections.Generic
import System.Threading

LongRunningTask = typing.Any

MomCrypto_Frontend__EventContainer_Callable = typing.TypeVar("MomCrypto_Frontend__EventContainer_Callable")
MomCrypto_Frontend__EventContainer_ReturnType = typing.TypeVar("MomCrypto_Frontend__EventContainer_ReturnType")


class ByteArrayEqualsComparer(System.Object, System.Collections.Generic.IEqualityComparer[typing.List[int]]):
    """This class has no documentation."""

    Instance: MomCrypto.Frontend.ByteArrayEqualsComparer = ...

    def Equals(self, x: typing.List[int], y: typing.List[int]) -> bool:
        ...

    def GetHashCode(self, obj: typing.List[int]) -> int:
        ...


class FrontendConnector(LongRunningTask):
    """This class has no documentation."""

    @property
    def pollTimerInterval(self) -> datetime.timedelta:
        """This field is protected."""
        ...

    @pollTimerInterval.setter
    def pollTimerInterval(self, value: datetime.timedelta):
        """This field is protected."""
        ...

    @property
    def PerMaxSendBlock(self) -> int:
        ...

    @PerMaxSendBlock.setter
    def PerMaxSendBlock(self, value: int):
        ...

    @property
    def PerMaxRecvBlock(self) -> int:
        ...

    @PerMaxRecvBlock.setter
    def PerMaxRecvBlock(self, value: int):
        ...

    @property
    def Address(self) -> str:
        ...

    @property
    def DebugMode(self) -> bool:
        ...

    @property
    def OnDisconnected(self) -> _EventContainer[typing.Callable[[MomCrypto.Frontend.FrontendConnector, int], None], None]:
        ...

    @OnDisconnected.setter
    def OnDisconnected(self, value: _EventContainer[typing.Callable[[MomCrypto.Frontend.FrontendConnector, int], None], None]):
        ...

    @property
    def OnSocketConnected(self) -> _EventContainer[typing.Callable[[MomCrypto.Frontend.FrontendConnector], None], None]:
        ...

    @OnSocketConnected.setter
    def OnSocketConnected(self, value: _EventContainer[typing.Callable[[MomCrypto.Frontend.FrontendConnector], None], None]):
        ...

    @property
    def ReceiveQueue(self) -> typing.Any:
        ...

    @property
    def SendQueue(self) -> typing.Any:
        ...

    def __init__(self, address: str, logger: typing.Any, debugMode: bool = False) -> None:
        ...

    def OnDealerConnected(self, sender: typing.Any, e: typing.Any) -> None:
        """This method is protected."""
        ...

    def OnDealerDisconnected(self, sender: typing.Any, e: typing.Any) -> None:
        """This method is protected."""
        ...

    def Run(self, ct: System.Threading.CancellationToken) -> None:
        """This method is protected."""
        ...


class FrontendEvent(System.Object):
    """This class has no documentation."""

    MaxMsgSize: int = ...

    @property
    def Identity(self) -> typing.List[int]:
        ...

    @Identity.setter
    def Identity(self, value: typing.List[int]):
        ...

    @property
    def MsgData(self) -> System.Collections.Generic.List[typing.List[int]]:
        ...

    @property
    def MsgSize(self) -> System.Collections.Generic.List[int]:
        ...

    @overload
    def __init__(self, identity: typing.List[int] = None) -> None:
        ...

    @overload
    def __init__(self, identity: typing.List[int], e: MomCrypto.Frontend.FrontendEvent) -> None:
        ...


class FrontendListener(LongRunningTask):
    """This class has no documentation."""

    @property
    def PerMaxSendBlock(self) -> int:
        ...

    @PerMaxSendBlock.setter
    def PerMaxSendBlock(self, value: int):
        ...

    @property
    def PerMaxRecvBlock(self) -> int:
        ...

    @PerMaxRecvBlock.setter
    def PerMaxRecvBlock(self, value: int):
        ...

    @property
    def pollTimerInterval(self) -> datetime.timedelta:
        """This field is protected."""
        ...

    @pollTimerInterval.setter
    def pollTimerInterval(self, value: datetime.timedelta):
        """This field is protected."""
        ...

    @property
    def Address(self) -> str:
        ...

    @property
    def DebugMode(self) -> bool:
        ...

    @property
    def ReceiveQueue(self) -> typing.Any:
        ...

    @property
    def SendQueue(self) -> typing.Any:
        ...

    def __init__(self, address: str, logger: typing.Any, debugMode: bool = False) -> None:
        ...

    def Run(self, ct: System.Threading.CancellationToken) -> None:
        """This method is protected."""
        ...


class IdentityEqualsComparer(System.Object, System.Collections.Generic.IEqualityComparer[typing.List[int]]):
    """This class has no documentation."""

    Instance: MomCrypto.Frontend.IdentityEqualsComparer = ...

    def Equals(self, x: typing.List[int], y: typing.List[int]) -> bool:
        ...

    def GetHashCode(self, buffer: typing.List[int]) -> int:
        ...


class NetMQHelper(System.Object):
    """This class has no documentation."""

    @staticmethod
    def GetSocketHandle(socket: typing.Any) -> int:
        ...


class _EventContainer(typing.Generic[MomCrypto_Frontend__EventContainer_Callable, MomCrypto_Frontend__EventContainer_ReturnType]):
    """This class is used to provide accurate autocomplete on events and cannot be imported."""

    def __call__(self, *args: typing.Any, **kwargs: typing.Any) -> MomCrypto_Frontend__EventContainer_ReturnType:
        """Fires the event."""
        ...

    def __iadd__(self, item: MomCrypto_Frontend__EventContainer_Callable) -> None:
        """Registers an event handler."""
        ...

    def __isub__(self, item: MomCrypto_Frontend__EventContainer_Callable) -> None:
        """Unregisters an event handler."""
        ...





from typing import overload
import typing

import QuantConnect
import QuantConnect.Algorithm.CSharp.LiveStrategy.DataType
import QuantConnect.Algorithm.CSharp.qlnet.tools
import QuantConnect.Data.Market
import QuantConnect.Interfaces
import QuantConnect.Orders
import QuantConnect.Securities
import QuantConnect.Statistics
import System
import System.Collections.Concurrent


class DeltaTrader(System.Object):
    """This class has no documentation."""

    @property
    def Underlying(self) -> QuantConnect.Algorithm.CSharp.LiveStrategy.DataType.Underlying:
        ...

    @property
    def CryptoSymbol(self) -> QuantConnect.Symbol:
        ...

    @property
    def FuturesSymbol(self) -> QuantConnect.Symbol:
        ...

    def __init__(self, underlying: QuantConnect.Algorithm.CSharp.LiveStrategy.DataType.Underlying, market: str, marketOrder: typing.Any, limitOrder: typing.Any, stopLimitOrder: typing.Any, getOpenOrders: typing.Any, getOrderTicket: typing.Any, getOrderById: typing.Any, futureSymbol: typing.Union[QuantConnect.Symbol, str], cryptoSymbol: typing.Union[QuantConnect.Symbol, str], enableMarginTrading: bool = False) -> None:
        ...

    def CreateClosingTwap(self, tradeVolume: float) -> None:
        ...

    def CreateTwaps(self, targetDelta: float, securities: QuantConnect.Securities.SecurityManager, cashBook: QuantConnect.Securities.CashBook, time: typing.Any, balanceNow: float, quoteCurrencyForSpotOnly: float, actionRequiredMessages: typing.Any, response: typing.Optional[typing.Any]) -> typing.Union[None, typing.Any]:
        ...

    def GetLeftTwapVolume(self, securities: QuantConnect.Securities.SecurityManager) -> float:
        ...

    def IsClosing(self) -> bool:
        ...

    def IsTrading(self) -> bool:
        ...

    def ManageOrders(self, utcNow: typing.Any) -> None:
        ...

    def ProcessOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        ...

    def TradeClosingTwap(self, time: typing.Any, security: QuantConnect.Securities.Security, response: typing.Optional[str]) -> typing.Union[bool, str]:
        ...

    def TradeTwaps(self, time: typing.Any, securities: QuantConnect.Securities.SecurityManager) -> bool:
        ...


class DeribitOptionTrader(System.Object):
    """This class has no documentation."""

    class PriceType(System.Enum):
        """This class has no documentation."""

        MidPrice = 0

        QuarterPrice = 1

        CounterPartyPrice = 2

    @property
    def Contract(self) -> QuantConnect.Data.Market.OptionContract:
        ...

    @property
    def Quantity(self) -> float:
        ...

    @Quantity.setter
    def Quantity(self, value: float):
        ...

    @property
    def Status(self) -> int:
        """This property contains the int value of a member of the QuantConnect.Statistics.OptionStatus enum."""
        ...

    @Status.setter
    def Status(self, value: int):
        """This property contains the int value of a member of the QuantConnect.Statistics.OptionStatus enum."""
        ...

    @property
    def OrderId(self) -> int:
        ...

    @OrderId.setter
    def OrderId(self, value: int):
        ...

    @property
    def CashDelta(self) -> float:
        ...

    @CashDelta.setter
    def CashDelta(self, value: float):
        ...

    @property
    def TradePrice(self) -> float:
        ...

    @TradePrice.setter
    def TradePrice(self, value: float):
        ...

    @property
    def HedgeRange(self) -> float:
        ...

    @HedgeRange.setter
    def HedgeRange(self, value: float):
        ...

    @property
    def LastCheckTime(self) -> typing.Any:
        ...

    @LastCheckTime.setter
    def LastCheckTime(self, value: typing.Any):
        ...

    @property
    def StartHedgeTime(self) -> typing.Any:
        ...

    @StartHedgeTime.setter
    def StartHedgeTime(self, value: typing.Any):
        ...

    @overload
    def __init__(self, contract: QuantConnect.Data.Market.OptionContract, quantity: float, midPrice: float, markPrice: float, bidPrice: float, askPrice: float, minPrice: float, lotSize: float, transactions: QuantConnect.Securities.SecurityTransactionManager, dingKey: str, dingToken: str, limitOrder: typing.Any, hedgeRange: float, sliceTime: typing.Any, minPriceVariation: float) -> None:
        ...

    @overload
    def __init__(self, contract: QuantConnect.Data.Market.OptionContract, quantity: float, transactions: QuantConnect.Securities.SecurityTransactionManager, status: QuantConnect.Statistics.OptionStatus, tradedVolume: float, lotSize: float, limitOrder: typing.Any, dingKey: str, dingToken: str, hedgeRange: float, sliceTime: typing.Any, minPriceVariation: float) -> None:
        ...

    def CalculateDelta(self, underlyingPrice: float, q: float, r: float, sigma: float, sliceTime: typing.Any, hedgeAll: bool) -> float:
        ...

    def CancelOrder(self, sliceTime: typing.Any) -> None:
        ...

    def GetTradedVolume(self) -> float:
        ...

    def NeedHedge(self) -> bool:
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        ...

    def SetPrices(self, midPrice: float, markPrice: float, bidPrice: float, askPrice: float, minPrice: float) -> None:
        ...

    def ToString(self) -> str:
        ...

    def TradeNext(self) -> None:
        ...

    def TryAdd(self, option: QuantConnect.Algorithm.CSharp.qlnet.tools.DeribitOptionTrader) -> bool:
        ...


class MyOrder(System.Object):
    """This class has no documentation."""

    @property
    def TradeSymbol(self) -> QuantConnect.Symbol:
        ...

    @property
    def TargetVolume(self) -> float:
        ...

    @property
    def FilledVolume(self) -> float:
        ...

    @FilledVolume.setter
    def FilledVolume(self, value: float):
        ...

    @property
    def OrderType(self) -> int:
        """This property contains the int value of a member of the QuantConnect.Orders.OrderType enum."""
        ...

    @property
    def OrderStatus(self) -> int:
        """This property contains the int value of a member of the QuantConnect.Orders.OrderStatus enum."""
        ...

    @OrderStatus.setter
    def OrderStatus(self, value: int):
        """This property contains the int value of a member of the QuantConnect.Orders.OrderStatus enum."""
        ...

    @property
    def Finished(self) -> bool:
        ...

    @Finished.setter
    def Finished(self, value: bool):
        ...

    @property
    def OrderID(self) -> int:
        ...

    @OrderID.setter
    def OrderID(self, value: int):
        ...

    @property
    def StopPrice(self) -> float:
        ...

    @property
    def LimitPrice(self) -> float:
        ...

    @property
    def OrderProperties(self) -> QuantConnect.Interfaces.IOrderProperties:
        ...

    def __init__(self, tradeSymbol: typing.Union[QuantConnect.Symbol, str], targetVolume: float, orderType: QuantConnect.Orders.OrderType, limitPrice: float = 0, stopPrice: float = 0) -> None:
        ...


class OrderManager(System.Object):
    """This class has no documentation."""

    @property
    def LocalOrders(self) -> System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Algorithm.CSharp.qlnet.tools.MyOrder]:
        ...

    @LocalOrders.setter
    def LocalOrders(self, value: System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Algorithm.CSharp.qlnet.tools.MyOrder]):
        ...

    @property
    def BrokerOrders(self) -> System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Orders.Order]:
        ...

    @BrokerOrders.setter
    def BrokerOrders(self, value: System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Orders.Order]):
        ...

    @property
    def TempOrders(self) -> System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Orders.Order]:
        ...

    @TempOrders.setter
    def TempOrders(self, value: System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Orders.Order]):
        ...

    def __init__(self, coinSymbol: typing.Union[QuantConnect.Symbol, str], marketOrder: typing.Any, limitOrder: typing.Any, stopLimitOrder: typing.Any, getOpenOrders: typing.Any, getOrderTicket: typing.Any, getOrderById: typing.Any) -> None:
        """创建订单管理实例"""
        ...

    def AddOrder(self, tradeSymbol: typing.Union[QuantConnect.Symbol, str], targetVolume: float, orderType: QuantConnect.Orders.OrderType, limitPrice: float = 0, stopPrice: float = 0) -> int:
        """添加期权order"""
        ...

    def CheckOrder(self) -> bool:
        """检查策略记录的订单和broker返回的订单是否一致"""
        ...

    def HasOpenOrder(self) -> bool:
        """检查是否有open order"""
        ...

    def ManageOrder(self, now: typing.Any) -> None:
        """处理所有的order"""
        ...

    def SubmitOrder(self, order: QuantConnect.Algorithm.CSharp.qlnet.tools.MyOrder) -> int:
        """提交期货order"""
        ...

    def UpdateOrder(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """用orderevent对order信息进行更新"""
        ...


class OrderMessage(System.Object):
    """This class has no documentation."""

    def __init__(self) -> None:
        ...

    def SendMessage(self, message: str, key: str, messageKey: str = ...) -> None:
        ...


class PositionManager(System.Object):
    """This class has no documentation."""

    @property
    def Symbol(self) -> QuantConnect.Symbol:
        ...

    @Symbol.setter
    def Symbol(self, value: QuantConnect.Symbol):
        ...

    def __init__(self, virtualPosition: float, symbol: typing.Union[QuantConnect.Symbol, str]) -> None:
        """创建仓位管理实例"""
        ...

    def AddPosition(self, vol: float) -> None:
        """在添加order时增加虚拟持仓"""
        ...

    def CheckPosition(self) -> bool:
        """检查策略记录的订单和broker返回的订单是否一致"""
        ...

    def ManagePosition(self, currentPosition: float) -> None:
        """处理现有的position"""
        ...

    def UpdatePosition(self, orderEvent: QuantConnect.Orders.OrderEvent, order: QuantConnect.Orders.Order) -> None:
        """用orderevent对position信息进行更新"""
        ...


class Utils(System.Object):
    """This class has no documentation."""

    class MoveStrikeType(System.Enum):
        """This class has no documentation."""

        MoveDownCashGamma = -1

        MoveBothCashGamma = 0

        MoveUpCashGamma = 1

    class OptionParityTradeMode(System.Enum):
        """This class has no documentation."""

        BuyBoth = 0

        OnlyBuyOTM = 1

        OnlyBuyITM = 2

    @staticmethod
    def CalculateMinOptionCost(mode: QuantConnect.Statistics.MinOptionCostMode, t2MDays: float, costMultiplier: float, underlyingPrice: float, strike: float, sigmaAddition: float, sigma: float) -> float:
        ...

    @staticmethod
    def RoundTradeVolume(tradeVolume: float, lotSize: float) -> float:
        ...

    @staticmethod
    def Sqrt(x: float, epsilon: float = ...) -> float:
        ...

    @staticmethod
    def Var(v: typing.Any) -> float:
        ...





from typing import overload
import datetime
import typing

import FtxApi
import FtxApi.Rest
import FtxApi.Rest.Enums
import FtxApi.Rest.Models
import FtxApi.Rest.Models.LeveragedTokens
import FtxApi.Rest.Models.Markets
import System
import System.Collections.Generic
import System.Threading.Tasks


class FtxRestApi(System.Object):
    """This class has no documentation."""

    def __init__(self, client: FtxApi.Client) -> None:
        ...

    def AcceptRequestQuoteAsync(self, quoteId: int) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.OptionQuote]]:
        ...

    def CancelAllOrdersAsync(self, instrument: str) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[str]]:
        ...

    def CancelOrderAsync(self, id: str) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[str]]:
        ...

    def CancelOrderByClientIdAsync(self, clientOrderId: str) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[str]]:
        ...

    def CancelQuoteRequestAsync(self, requestId: int) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.QuoteRequest]]:
        ...

    def ChangeAccountLeverageAsync(self, leverage: int) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.AccountLeverage]:
        ...

    def CreateQuoteRequestAsync(self, side: FtxApi.Rest.Enums.SideType, type: FtxApi.Rest.Enums.OptionType, strike: float, size: float, expiry: typing.Optional[datetime.datetime] = None, limitPrice: typing.Optional[float] = None, hideLimitPrice: typing.Optional[bool] = None, requestExpiry: typing.Optional[datetime.datetime] = None, counterPartyId: typing.Optional[int] = None) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.QuoteRequest]]:
        ...

    def GetAccountInfoAsync(self) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.AccountInfo]]:
        ...

    def GetAllFuturesAsync(self) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.Future]]]:
        ...

    def GetBalancesAsync(self) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.Balance]]]:
        ...

    def GetCoinAsync(self) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.Coin]]]:
        ...

    def GetCoinsAsync(self) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.Coin]]]:
        ...

    def GetDepositAddressAsync(self, coin: str) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.DepositAddress]]:
        ...

    def GetDepositHistoryAsync(self) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.DepositHistory]]]:
        ...

    @overload
    def GetFillsAsync(self, market: str, limit: int, start: typing.Union[datetime.datetime, datetime.date], end: typing.Union[datetime.datetime, datetime.date]) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.Fill]]]:
        ...

    @overload
    def GetFillsAsync(self, limit: int, start: typing.Union[datetime.datetime, datetime.date]) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.Fill]]]:
        ...

    def GetFundingPaymentAsync(self, start: typing.Union[datetime.datetime, datetime.date], end: typing.Union[datetime.datetime, datetime.date]) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.FundingPayment]]]:
        ...

    def GetFundingRatesAsync(self, start: typing.Union[datetime.datetime, datetime.date], end: typing.Union[datetime.datetime, datetime.date]) -> System.Threading.Tasks.Task[System.Collections.Generic.List[FtxApi.Rest.Models.FundingRate]]:
        ...

    def GetFutureAsync(self, future: str) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.Future]]:
        ...

    def GetFutureStatsAsync(self, future: str) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.FutureStats]]:
        ...

    def GetHistoricalPricesAsync(self, marketName: str, resolution: int, start: typing.Union[datetime.datetime, datetime.date], end: typing.Union[datetime.datetime, datetime.date]) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.Candle]]]:
        ...

    def GetLeveragedTokenBalancesAsync(self) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.LeveragedTokens.LeveragedTokenBalance]]]:
        ...

    def GetLeveragedTokenCreationListAsync(self) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.LeveragedTokens.LeveragedTokenCreation]]]:
        ...

    def GetLeveragedTokenRedemptionListAsync(self) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.LeveragedTokens.LeveragedTokenRedemptionRequest]]]:
        ...

    def GetLeveragedTokensListAsync(self) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.LeveragedTokens.LeveragedToken]]]:
        ...

    def GetMarketOrderBookAsync(self, marketName: str, depth: int = 20) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.Markets.Orderbook]]:
        ...

    def GetMarketsAsync(self) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.Markets.Market]]]:
        ...

    def GetMarketTradesAsync(self, marketName: str, limit: int, start: typing.Union[datetime.datetime, datetime.date], end: typing.Union[datetime.datetime, datetime.date]) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.Markets.Trade]]]:
        ...

    def GetMyQuoteRequestsAsync(self) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.QuoteRequest]]]:
        ...

    def GetOpenOrdersAsync(self, instrument: str = None) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.Order]]]:
        ...

    def GetOptionsPositionsAsync(self) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.OptionPosition]]]:
        ...

    def GetOrderFillsAsync(self, orderId: str) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.Fill]]]:
        ...

    def GetOrderStatusAsync(self, id: str) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.Order]]:
        ...

    def GetOrderStatusByClientIdAsync(self, clientOrderId: str) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.Order]]:
        ...

    def GetPositionsAsync(self) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.Position]]]:
        ...

    def GetQuoteRequestsAsync(self) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.QuoteRequest]]]:
        ...

    def GetRequestQuotesAsync(self, requestId: int) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.OptionQuote]]]:
        ...

    def GetSingleMarketsAsync(self, marketName: str) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.Markets.Market]]:
        ...

    def GetTokenInfoAsync(self, tokenName: str) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.LeveragedTokens.LeveragedToken]]:
        ...

    def GetWithdrawalHistoryAsync(self) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[System.Collections.Generic.List[FtxApi.Rest.Models.WithdrawalHistory]]]:
        ...

    def PlaceOrderAsync(self, instrument: str, side: FtxApi.Rest.Enums.SideType, price: float, orderType: FtxApi.Rest.Enums.OrderType, amount: float, clientId: str = None, ioc: bool = False, reduceOnly: typing.Optional[bool] = False) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.Order]]:
        ...

    def PlaceStopOrderAsync(self, instrument: str, side: FtxApi.Rest.Enums.SideType, type: FtxApi.Rest.Enums.TriggerType, triggerPrice: float, amount: float, orderPrice: typing.Optional[float] = None, reduceOnly: typing.Optional[bool] = None, retryUntilFilled: typing.Optional[bool] = None) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.TriggerOrder]]:
        ...

    def RequestLeveragedTokenCreationAsync(self, tokenName: str, size: float) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.LeveragedTokens.LeveragedTokenCreationRequest]]:
        ...

    def RequestLeveragedTokenRedemptionAsync(self, tokenName: str, size: float) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.LeveragedTokens.LeveragedTokenRedemption]]:
        ...

    def RequestWithdrawalAsync(self, coin: str, size: float, address: str, tag: str, pass: str, code: str) -> System.Threading.Tasks.Task[FtxApi.Rest.Models.FtxResult[FtxApi.Rest.Models.WithdrawalHistory]]:
        ...


class FtxRestApiFactory(System.Object):
    """This class has no documentation."""

    @staticmethod
    def CreateClient(apiKey: str, apiSecret: str) -> FtxApi.Rest.FtxRestApi:
        ...





from typing import overload
import datetime
import typing

import QuantConnect
import QuantConnect.ABL
import QuantConnect.Interfaces
import QuantConnect.Orders
import QuantConnect.Securities
import System
import System.Collections.Generic

QuantConnect_ABL_QueryExchangeResult_T = typing.TypeVar("QuantConnect_ABL_QueryExchangeResult_T")
QuantConnect_ABL_ServerCallResult_T = typing.TypeVar("QuantConnect_ABL_ServerCallResult_T")
QuantConnect_ABL__EventContainer_Callable = typing.TypeVar("QuantConnect_ABL__EventContainer_Callable")
QuantConnect_ABL__EventContainer_ReturnType = typing.TypeVar("QuantConnect_ABL__EventContainer_ReturnType")


class StocksSplitInfo(System.Object):
    """"""

    @property
    def SplitFactor(self) -> float:
        ...

    @SplitFactor.setter
    def SplitFactor(self, value: float):
        ...

    @property
    def SplitDate(self) -> datetime.datetime:
        ...

    @SplitDate.setter
    def SplitDate(self, value: datetime.datetime):
        ...

    @property
    def Symbol(self) -> str:
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        ...


class SplitManager(System.Object):
    """This class has no documentation."""

    def __init__(self, path: str) -> None:
        ...

    def GetSplit(self, symbol: str, date: typing.Union[datetime.datetime, datetime.date]) -> QuantConnect.ABL.StocksSplitInfo:
        ...


class StocksDividendInfo(System.Object):
    """This class has no documentation."""

    @property
    def PayDate(self) -> datetime.datetime:
        ...

    @PayDate.setter
    def PayDate(self, value: datetime.datetime):
        ...

    @property
    def ExDividendDate(self) -> datetime.datetime:
        ...

    @ExDividendDate.setter
    def ExDividendDate(self, value: datetime.datetime):
        ...

    @property
    def DividendCash(self) -> float:
        ...

    @DividendCash.setter
    def DividendCash(self, value: float):
        ...


class StocksDividend(System.Object):
    """This class has no documentation."""

    def __init__(self, content: str) -> None:
        ...

    def GetDividend(self, date: typing.Union[datetime.datetime, datetime.date]) -> QuantConnect.ABL.StocksDividendInfo:
        ...


class DividendManager(System.Object):
    """This class has no documentation."""

    def __init__(self, path: str) -> None:
        ...

    def GetDividend(self, symbol: str, date: typing.Union[datetime.datetime, datetime.date]) -> QuantConnect.ABL.StocksDividendInfo:
        ...


class EtfOptionChange(System.Object):
    """This class has no documentation."""

    @property
    def Underlying(self) -> str:
        ...

    @Underlying.setter
    def Underlying(self, value: str):
        ...

    @property
    def ExpiryDate(self) -> datetime.datetime:
        ...

    @ExpiryDate.setter
    def ExpiryDate(self, value: datetime.datetime):
        ...

    @property
    def NewExpiryDate(self) -> datetime.datetime:
        ...

    @NewExpiryDate.setter
    def NewExpiryDate(self, value: datetime.datetime):
        ...

    @property
    def ActionDate(self) -> datetime.datetime:
        ...

    @ActionDate.setter
    def ActionDate(self, value: datetime.datetime):
        ...

    def __init__(self, underlying: str, expiryDate: typing.Union[datetime.datetime, datetime.date], newExpiryDate: typing.Union[datetime.datetime, datetime.date], actionDate: typing.Union[datetime.datetime, datetime.date]) -> None:
        ...


class EtfOptionChangeManager(System.Object):
    """This class has no documentation."""

    def __init__(self, file: str) -> None:
        ...

    def AddEtfOptionChange(self, etfOptionChange: QuantConnect.ABL.EtfOptionChange) -> None:
        ...

    def GetEtfOptionChange(self, underlying: str, actionDate: typing.Union[datetime.datetime, datetime.date]) -> System.Collections.Generic.List[QuantConnect.ABL.EtfOptionChange]:
        ...

    def GetEtfOptionChanges(self) -> System.Collections.Generic.Dictionary[str, System.Collections.Generic.List[QuantConnect.ABL.EtfOptionChange]]:
        ...


class AblSymbolDatabase(System.Object):
    """AblSymbolDatabase"""

    ExchangeUtcTimeTicks: int = 0
    """ExchangeUtcTimeTicks"""

    IsWarmingUp: bool = True
    """IsWarmingUp"""

    @staticmethod
    def FromDataFolder() -> None:
        """FromDataFolder"""
        ...

    @staticmethod
    def GetAdjustFactor(symbol: typing.Union[QuantConnect.Symbol, str], date: typing.Union[datetime.datetime, datetime.date]) -> float:
        """GetAdjustFactor"""
        ...

    @staticmethod
    def GetDividend(symbol: typing.Union[QuantConnect.Symbol, str], date: typing.Union[datetime.datetime, datetime.date]) -> QuantConnect.ABL.StocksDividendInfo:
        """GetDividend"""
        ...

    @staticmethod
    def GetDividendTicker(symbol: typing.Union[QuantConnect.Symbol, str]) -> str:
        """GetDividendTicker"""
        ...

    @staticmethod
    def GetExpiryChangeTicker(symbol: typing.Union[QuantConnect.Symbol, str]) -> str:
        """GetExpiryChangeTicker"""
        ...

    @staticmethod
    def GetForwardAdjustFactor(symbol: typing.Union[QuantConnect.Symbol, str], date: typing.Union[datetime.datetime, datetime.date]) -> float:
        """GetForwardAdjustFactor"""
        ...

    @staticmethod
    def GetOptionChange(symbol: typing.Union[QuantConnect.Symbol, str], date: typing.Union[datetime.datetime, datetime.date]) -> System.Collections.Generic.List[QuantConnect.ABL.EtfOptionChange]:
        """GetOptionChange"""
        ...

    @staticmethod
    def GetOptionChangedTicker(symbol: typing.Union[QuantConnect.Symbol, str], date: typing.Union[datetime.datetime, datetime.date]) -> str:
        """GetOptionChangedTicker"""
        ...

    @staticmethod
    def GetSplit(symbol: typing.Union[QuantConnect.Symbol, str], date: typing.Union[datetime.datetime, datetime.date]) -> QuantConnect.ABL.StocksSplitInfo:
        """GetSplit"""
        ...

    @staticmethod
    def IsChinaEtf(symbol: typing.Union[QuantConnect.Symbol, str]) -> bool:
        """IsChinaEtf"""
        ...

    @staticmethod
    def IsChinaIndex(symbol: typing.Union[QuantConnect.Symbol, str]) -> bool:
        """IsChinaIndex"""
        ...

    @staticmethod
    def IsChinaStocks(symbol: typing.Union[QuantConnect.Symbol, str]) -> bool:
        """IsChinaStocks"""
        ...

    @staticmethod
    def IsDividendTicker(symbol: str) -> bool:
        """IsDividendTicker"""
        ...

    @staticmethod
    def IsExpiryChangeTicker(symbol: str) -> bool:
        """IsExpiryChangeTicker"""
        ...

    @staticmethod
    def LockDividendSymbol(underlying: typing.Union[QuantConnect.Symbol, str], date: typing.Union[datetime.datetime, datetime.date]) -> None:
        """LockDividendSymbol"""
        ...

    @staticmethod
    def UnlockDividendSymbol(underlying: typing.Union[QuantConnect.Symbol, str], date: typing.Union[datetime.datetime, datetime.date]) -> None:
        """UnlockDividendSymbol"""
        ...

    @staticmethod
    def WaitForDividendSymbolLock(underlying: typing.Union[QuantConnect.Symbol, str], date: typing.Union[datetime.datetime, datetime.date]) -> int:
        """WaitForDividendSymbolLock"""
        ...


class ExchangeQueryInfo(System.Object):
    """"""

    @property
    def QueryType(self) -> int:
        """
        QueryType
        
        This property contains the int value of a member of the QuantConnect.ABL.ExchangeQueryType enum.
        """
        ...

    @QueryType.setter
    def QueryType(self, value: int):
        """
        QueryType
        
        This property contains the int value of a member of the QuantConnect.ABL.ExchangeQueryType enum.
        """
        ...

    @property
    def UserId(self) -> str:
        """UserId"""
        ...

    @UserId.setter
    def UserId(self, value: str):
        """UserId"""
        ...

    @property
    def RequestId(self) -> str:
        """RequestId"""
        ...

    @RequestId.setter
    def RequestId(self, value: str):
        """RequestId"""
        ...


class UserTransferType(System.Enum):
    """UserTransferType"""

    Spot2UmFuture = 0
    """Spot2UmFuture"""

    UmFuture2Spot = 1
    """UmFuture2Spot"""


class TransferInfo(System.Object):
    """"""

    @property
    def Amount(self) -> float:
        """Amount"""
        ...

    @property
    def TransferType(self) -> int:
        """
        TransferType
        
        This property contains the int value of a member of the QuantConnect.ABL.UserTransferType enum.
        """
        ...

    @property
    def Currency(self) -> str:
        """Currency"""
        ...

    def __init__(self, amount: float, type: QuantConnect.ABL.UserTransferType, currency: str) -> None:
        """"""
        ...


class QueryExchangeResult(typing.Generic[QuantConnect_ABL_QueryExchangeResult_T], System.Object):
    """"""

    @property
    def Data(self) -> QuantConnect_ABL_QueryExchangeResult_T:
        ...

    @Data.setter
    def Data(self, value: QuantConnect_ABL_QueryExchangeResult_T):
        ...

    @property
    def RequestId(self) -> str:
        ...

    @RequestId.setter
    def RequestId(self, value: str):
        ...

    @property
    def IsLast(self) -> bool:
        ...

    @IsLast.setter
    def IsLast(self, value: bool):
        ...


class RiskDegreeInfo(System.Object):
    """"""

    @property
    def RiskDegree(self) -> float:
        ...

    @RiskDegree.setter
    def RiskDegree(self, value: float):
        ...

    @property
    def UserRiskDegree(self) -> float:
        ...

    @UserRiskDegree.setter
    def UserRiskDegree(self, value: float):
        ...


class ServerCallResult(typing.Generic[QuantConnect_ABL_ServerCallResult_T], System.Object):
    """"""

    @property
    def Data(self) -> QuantConnect_ABL_ServerCallResult_T:
        ...

    @Data.setter
    def Data(self, value: QuantConnect_ABL_ServerCallResult_T):
        ...

    @property
    def ErrorMessage(self) -> str:
        ...

    @ErrorMessage.setter
    def ErrorMessage(self, value: str):
        ...

    @property
    def Success(self) -> bool:
        ...

    @Success.setter
    def Success(self, value: bool):
        ...

    def __init__(self, data: QuantConnect_ABL_ServerCallResult_T) -> None:
        """"""
        ...

    @staticmethod
    def Failure(data: QuantConnect_ABL_ServerCallResult_T, error: str) -> QuantConnect.ABL.ServerCallResult[QuantConnect_ABL_ServerCallResult_T]:
        """"""
        ...


class AlgorithmBoost(System.Object):
    """Algorithm Boost"""

    @property
    def AlgorithmEnd(self) -> _EventContainer[typing.Callable[[], None], None]:
        ...

    @AlgorithmEnd.setter
    def AlgorithmEnd(self, value: _EventContainer[typing.Callable[[], None], None]):
        ...

    @property
    def SymbolListed(self) -> _EventContainer[typing.Callable[[System.Collections.Generic.IList[QuantConnect.Symbol]], None], None]:
        ...

    @SymbolListed.setter
    def SymbolListed(self, value: _EventContainer[typing.Callable[[System.Collections.Generic.IList[QuantConnect.Symbol]], None], None]):
        ...

    @property
    def SymbolDelisted(self) -> _EventContainer[typing.Callable[[System.Collections.Generic.IList[QuantConnect.Symbol]], None], None]:
        ...

    @SymbolDelisted.setter
    def SymbolDelisted(self, value: _EventContainer[typing.Callable[[System.Collections.Generic.IList[QuantConnect.Symbol]], None], None]):
        ...

    @property
    def QueryExchangeBalanceCompleted(self) -> _EventContainer[typing.Callable[[QuantConnect.ABL.QueryExchangeResult[QuantConnect.Securities.CashAmount]], None], None]:
        ...

    @QueryExchangeBalanceCompleted.setter
    def QueryExchangeBalanceCompleted(self, value: _EventContainer[typing.Callable[[QuantConnect.ABL.QueryExchangeResult[QuantConnect.Securities.CashAmount]], None], None]):
        ...

    @property
    def QueryExchangeHoldingCompleted(self) -> _EventContainer[typing.Callable[[QuantConnect.ABL.QueryExchangeResult[QuantConnect.Holding]], None], None]:
        ...

    @QueryExchangeHoldingCompleted.setter
    def QueryExchangeHoldingCompleted(self, value: _EventContainer[typing.Callable[[QuantConnect.ABL.QueryExchangeResult[QuantConnect.Holding]], None], None]):
        ...

    @property
    def QueryExchangeOpenOrderCompleted(self) -> _EventContainer[typing.Callable[[QuantConnect.ABL.QueryExchangeResult[QuantConnect.Orders.Order]], None], None]:
        ...

    @QueryExchangeOpenOrderCompleted.setter
    def QueryExchangeOpenOrderCompleted(self, value: _EventContainer[typing.Callable[[QuantConnect.ABL.QueryExchangeResult[QuantConnect.Orders.Order]], None], None]):
        ...

    @property
    def QueryBalanceCompleted(self) -> _EventContainer[typing.Callable[[QuantConnect.ABL.QueryExchangeResult[QuantConnect.Securities.CashAmount]], None], None]:
        ...

    @QueryBalanceCompleted.setter
    def QueryBalanceCompleted(self, value: _EventContainer[typing.Callable[[QuantConnect.ABL.QueryExchangeResult[QuantConnect.Securities.CashAmount]], None], None]):
        ...

    @property
    def QueryHoldingCompleted(self) -> _EventContainer[typing.Callable[[QuantConnect.ABL.QueryExchangeResult[QuantConnect.Holding]], None], None]:
        ...

    @QueryHoldingCompleted.setter
    def QueryHoldingCompleted(self, value: _EventContainer[typing.Callable[[QuantConnect.ABL.QueryExchangeResult[QuantConnect.Holding]], None], None]):
        ...

    @property
    def QueryOpenOrderCompleted(self) -> _EventContainer[typing.Callable[[QuantConnect.ABL.QueryExchangeResult[QuantConnect.Orders.Order]], None], None]:
        ...

    @QueryOpenOrderCompleted.setter
    def QueryOpenOrderCompleted(self, value: _EventContainer[typing.Callable[[QuantConnect.ABL.QueryExchangeResult[QuantConnect.Orders.Order]], None], None]):
        ...

    @property
    def RiskDegreeChanged(self) -> _EventContainer[typing.Callable[[QuantConnect.ABL.RiskDegreeInfo], None], None]:
        ...

    @RiskDegreeChanged.setter
    def RiskDegreeChanged(self, value: _EventContainer[typing.Callable[[QuantConnect.ABL.RiskDegreeInfo], None], None]):
        ...

    @property
    def TransferFinished(self) -> _EventContainer[typing.Callable[[QuantConnect.ABL.ServerCallResult[QuantConnect.ABL.TransferInfo]], None], None]:
        ...

    @TransferFinished.setter
    def TransferFinished(self, value: _EventContainer[typing.Callable[[QuantConnect.ABL.ServerCallResult[QuantConnect.ABL.TransferInfo]], None], None]):
        ...

    @property
    def TransferAction(self) -> typing.Callable[[QuantConnect.ABL.TransferInfo], None]:
        ...

    @TransferAction.setter
    def TransferAction(self, value: typing.Callable[[QuantConnect.ABL.TransferInfo], None]):
        ...

    @property
    def QueryExchangeAction(self) -> typing.Callable[[QuantConnect.ABL.ExchangeQueryInfo], None]:
        ...

    @QueryExchangeAction.setter
    def QueryExchangeAction(self, value: typing.Callable[[QuantConnect.ABL.ExchangeQueryInfo], None]):
        ...

    @property
    def CryptoSymbolProvider(self) -> QuantConnect.Interfaces.ICryptoChainProvider:
        ...

    @CryptoSymbolProvider.setter
    def CryptoSymbolProvider(self, value: QuantConnect.Interfaces.ICryptoChainProvider):
        ...

    @property
    def TradingIsReady(self) -> bool:
        """TradingIsReady"""
        ...

    @TradingIsReady.setter
    def TradingIsReady(self, value: bool):
        """TradingIsReady"""
        ...

    def RaiseAlgorithmEnd(self) -> None:
        """"""
        ...

    def RaiseOnQueryExchangeHolding(self, holding: QuantConnect.Holding, requestId: str, isLast: bool) -> None:
        """RaiseOnQueryExchangeHolding"""
        ...

    def RaiseQueryBalanceCompleted(self, balance: QuantConnect.Securities.CashAmount, requestId: str) -> None:
        """RaiseQueryBalanceCompleted"""
        ...

    def RaiseQueryExchangeBalanceCompleted(self, balance: QuantConnect.Securities.CashAmount, requestId: str, isLast: bool) -> None:
        """RaiseQueryExchangeBalanceCompleted"""
        ...

    def RaiseQueryExchangeOpenOrderCompleted(self, order: QuantConnect.Orders.Order, requestId: str, isLast: bool) -> None:
        """RaiseQueryExchangeOpenOrderCompleted"""
        ...

    def RaiseQueryHoldingCompleted(self, holding: QuantConnect.Holding, requestId: str) -> None:
        """RaiseQueryHolding"""
        ...

    def RaiseQueryOpenOrderCompleted(self, order: QuantConnect.Orders.Order, requestId: str) -> None:
        """RaiseQueryOpenOrderCompleted"""
        ...

    def RaiseRiskDegreeChanged(self, riskDegree: float, userRiskDegree: float) -> None:
        """RaiseRiskDegreeChanged"""
        ...

    def RaiseSymbolDelisted(self, symbols: System.Collections.Generic.IList[QuantConnect.Symbol]) -> None:
        """RaiseSymbolDelisted"""
        ...

    def RaiseSymbolListed(self, symbols: System.Collections.Generic.IList[QuantConnect.Symbol]) -> None:
        """RaiseSymbolListed"""
        ...

    def RaiseTransferFinished(self, amount: float, type: QuantConnect.ABL.UserTransferType, currency: str, error: str = None) -> None:
        """"""
        ...


class ExchangeQueryType(System.Enum):
    """"""

    OpenOrder = 1
    """OpenOrder"""

    Holding = 1
    """Holding"""

    CashBalance = 2
    """CashBalance"""

    ExchangeOpenOrder = 3
    """"""

    ExchangeHolding = 4
    """"""

    ExchangeCashBalance = 5
    """"""


class PriceFactorManager(System.Object):
    """PriceFactorManager"""

    def __init__(self, path: str) -> None:
        """PriceFactorManager"""
        ...

    def GetAdjustFactor(self, symbol: str, date: typing.Union[datetime.datetime, datetime.date]) -> float:
        """GetAdjustFactor"""
        ...

    def GetForwardAdjustFactor(self, symbol: str, date: typing.Union[datetime.datetime, datetime.date]) -> float:
        """GetForwardAdjustFactor"""
        ...


class StocksPriceFactor(System.Object):
    """StocksPriceFactor"""

    @property
    def LastFactor(self) -> float:
        ...

    @property
    def LastDate(self) -> datetime.datetime:
        ...

    @property
    def FirstDate(self) -> datetime.datetime:
        ...

    def __init__(self, file: str) -> None:
        """StocksPriceFactor"""
        ...

    def GetAdjustFactor(self, date: typing.Union[datetime.datetime, datetime.date]) -> float:
        ...

    def GetForwardAdjustFactor(self, date: typing.Union[datetime.datetime, datetime.date]) -> float:
        ...


class _EventContainer(typing.Generic[QuantConnect_ABL__EventContainer_Callable, QuantConnect_ABL__EventContainer_ReturnType]):
    """This class is used to provide accurate autocomplete on events and cannot be imported."""

    def __call__(self, *args: typing.Any, **kwargs: typing.Any) -> QuantConnect_ABL__EventContainer_ReturnType:
        """Fires the event."""
        ...

    def __iadd__(self, item: QuantConnect_ABL__EventContainer_Callable) -> None:
        """Registers an event handler."""
        ...

    def __isub__(self, item: QuantConnect_ABL__EventContainer_Callable) -> None:
        """Unregisters an event handler."""
        ...





from typing import overload
import datetime
import typing

import QuantConnect
import QuantConnect.Algorithm.CSharp.qlnet.AlgoTrade
import QuantConnect.Algorithm.CSharp.qlnet.tools
import QuantConnect.Orders
import QuantConnect.Securities
import System


class OptionTwap(System.Object):
    """This class has no documentation."""

    class PriceType(System.Enum):
        """This class has no documentation."""

        MidPrice = 0

        QuoterPrice = 1

        CounterPartyPrice = 2

    @property
    def Symbol(self) -> QuantConnect.Symbol:
        ...

    @property
    def OrderId(self) -> int:
        ...

    @OrderId.setter
    def OrderId(self, value: int):
        ...

    def __init__(self, symbol: typing.Union[QuantConnect.Symbol, str], targetVolume: float, transactions: QuantConnect.Securities.SecurityTransactionManager, totalSeconds: float, interval: float, bidPriceWeight: float, bidPrice: float, askPrice: float, minPrice: float, limitOrder: typing.Any, dingKey: str, dingToken: str, lotSize: float = 1) -> None:
        ...

    def IsFinished(self) -> bool:
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        ...

    def ToString(self) -> str:
        ...

    def UpdatePrices(self, bidPriceWeight: float, bidPrice: float, askPrice: float, minPrice: float) -> None:
        ...


class Twap(System.Object):
    """This class has no documentation."""

    @property
    def IsFinished(self) -> bool:
        ...

    @IsFinished.setter
    def IsFinished(self, value: bool):
        ...

    @property
    def Symbol(self) -> QuantConnect.Symbol:
        ...

    def __init__(self, om: QuantConnect.Algorithm.CSharp.qlnet.tools.OrderManager, symbol: typing.Union[QuantConnect.Symbol, str], target_vol: float, order_type: QuantConnect.Orders.OrderType, totalseconds: float, count: int) -> None:
        ...

    def CurrentLeftVolume(self) -> float:
        ...

    def TwapTrade(self, sliceTime: typing.Union[datetime.datetime, datetime.date], security: QuantConnect.Securities.Security, getOrderById: typing.Any) -> bool:
        ...

    def UpdateTwapStatus(self, security: QuantConnect.Securities.Security, getOrderById: typing.Any) -> None:
        ...





from typing import overload
import abc

import Calculators.IO
import System
import System.Collections.Generic

Calculators_IO_AggregatedDataLoader_FetchData_T = typing.TypeVar("Calculators_IO_AggregatedDataLoader_FetchData_T")
Calculators_IO_AggregatedDataWriter_ToCsv_T = typing.TypeVar("Calculators_IO_AggregatedDataWriter_ToCsv_T")
Calculators_IO_CsvDataLoader_FetchData_T = typing.TypeVar("Calculators_IO_CsvDataLoader_FetchData_T")
Calculators_IO_CsvDataWriter_ToCsv_T = typing.TypeVar("Calculators_IO_CsvDataWriter_ToCsv_T")
Calculators_IO_IDataLoader_FetchData_T = typing.TypeVar("Calculators_IO_IDataLoader_FetchData_T")
Calculators_IO_IDataWriter_ToCsv_T = typing.TypeVar("Calculators_IO_IDataWriter_ToCsv_T")


class IDataLoader(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def FetchData(self, location: str) -> System.Collections.Generic.IEnumerable[Calculators_IO_IDataLoader_FetchData_T]:
        ...


class AggregatedDataLoader(System.Object, Calculators.IO.IDataLoader):
    """This class has no documentation."""

    def FetchData(self, location: str) -> System.Collections.Generic.IEnumerable[Calculators_IO_AggregatedDataLoader_FetchData_T]:
        ...


class IDataWriter(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def ToCsv(self, data: System.Collections.Generic.IEnumerable[Calculators_IO_IDataWriter_ToCsv_T], path: str, append: bool) -> None:
        ...


class AggregatedDataWriter(System.Object, Calculators.IO.IDataWriter):
    """This class has no documentation."""

    def ToCsv(self, data: System.Collections.Generic.IEnumerable[Calculators_IO_AggregatedDataWriter_ToCsv_T], path: str, append: bool) -> None:
        ...


class CsvDataLoader(System.Object, Calculators.IO.IDataLoader):
    """This class has no documentation."""

    def FetchData(self, filePath: str) -> System.Collections.Generic.IEnumerable[Calculators_IO_CsvDataLoader_FetchData_T]:
        ...


class CsvDataWriter(System.Object, Calculators.IO.IDataWriter):
    """This class has no documentation."""

    def ToCsv(self, datas: System.Collections.Generic.IEnumerable[Calculators_IO_CsvDataWriter_ToCsv_T], path: str, append: bool) -> None:
        ...





from typing import overload
import datetime
import typing

import FtxApi.WebSocket.Models
import System
import System.Collections.Generic

FtxApi_WebSocket_Models_FtxWebsocketReceive = typing.Any

FtxApi_WebSocket_Models_FtxWebsocketReceive_T = typing.TypeVar("FtxApi_WebSocket_Models_FtxWebsocketReceive_T")
FtxApi_WebSocket_Models_DataAction_T = typing.TypeVar("FtxApi_WebSocket_Models_DataAction_T")


class FtxOrderBook(System.Object):
    """This class has no documentation."""

    @property
    def time(self) -> float:
        ...

    @time.setter
    def time(self, value: float):
        ...

    @property
    def checksum(self) -> int:
        ...

    @checksum.setter
    def checksum(self, value: int):
        ...

    @property
    def action(self) -> str:
        ...

    @action.setter
    def action(self, value: str):
        ...

    @property
    def bids(self) -> System.Collections.Generic.List[typing.List[typing.Optional[float]]]:
        ...

    @bids.setter
    def bids(self, value: System.Collections.Generic.List[typing.List[typing.Optional[float]]]):
        ...

    @property
    def asks(self) -> System.Collections.Generic.List[typing.List[typing.Optional[float]]]:
        ...

    @asks.setter
    def asks(self, value: System.Collections.Generic.List[typing.List[typing.Optional[float]]]):
        ...

    @property
    def id(self) -> str:
        ...

    @id.setter
    def id(self, value: str):
        ...

    def Copy(self) -> FtxApi.WebSocket.Models.FtxOrderBook:
        ...

    def GetTime(self) -> System.DateTimeOffset:
        ...


class FtxOrderBookHelper(System.Object):
    """This class has no documentation."""

    @staticmethod
    def GetFtxOrderBookPrice(array: typing.List[typing.Optional[float]]) -> typing.Optional[float]:
        ...

    @staticmethod
    def GetFtxOrderBookVolume(array: typing.List[typing.Optional[float]]) -> typing.Optional[float]:
        ...


class FtxTicker(System.Object):
    """This class has no documentation."""

    @property
    def bid(self) -> typing.Optional[float]:
        ...

    @bid.setter
    def bid(self, value: typing.Optional[float]):
        ...

    @property
    def ask(self) -> typing.Optional[float]:
        ...

    @ask.setter
    def ask(self, value: typing.Optional[float]):
        ...

    @property
    def bidSize(self) -> typing.Optional[float]:
        ...

    @bidSize.setter
    def bidSize(self, value: typing.Optional[float]):
        ...

    @property
    def askSize(self) -> typing.Optional[float]:
        ...

    @askSize.setter
    def askSize(self, value: typing.Optional[float]):
        ...

    @property
    def last(self) -> typing.Optional[float]:
        ...

    @last.setter
    def last(self, value: typing.Optional[float]):
        ...

    @property
    def time(self) -> float:
        ...

    @time.setter
    def time(self, value: float):
        ...

    @property
    def id(self) -> str:
        ...

    @id.setter
    def id(self, value: str):
        ...

    def GetTime(self) -> System.DateTimeOffset:
        ...


class FtxWebsocketReceive(typing.Generic[FtxApi_WebSocket_Models_FtxWebsocketReceive_T], FtxApi_WebSocket_Models_FtxWebsocketReceive):
    """This class has no documentation."""

    Partial: str = "partial"

    Update: str = "update"

    @property
    def Channel(self) -> str:
        ...

    @Channel.setter
    def Channel(self, value: str):
        ...

    @property
    def Type(self) -> str:
        ...

    @Type.setter
    def Type(self, value: str):
        ...

    @property
    def Market(self) -> str:
        ...

    @Market.setter
    def Market(self, value: str):
        ...

    @property
    def ErrorMessage(self) -> str:
        ...

    @ErrorMessage.setter
    def ErrorMessage(self, value: str):
        ...

    @property
    def Data(self) -> FtxApi_WebSocket_Models_FtxWebsocketReceive_T:
        ...

    @Data.setter
    def Data(self, value: FtxApi_WebSocket_Models_FtxWebsocketReceive_T):
        ...


class DataAction(typing.Generic[FtxApi_WebSocket_Models_DataAction_T], System.Object):
    """This class has no documentation."""

    @property
    def Data(self) -> FtxApi_WebSocket_Models_DataAction_T:
        ...

    @Data.setter
    def Data(self, value: FtxApi_WebSocket_Models_DataAction_T):
        ...

    @property
    def Action(self) -> str:
        ...

    @Action.setter
    def Action(self, value: str):
        ...


class MarketState(System.Object):
    """This class has no documentation."""

    class FutureState(System.Object):
        """This class has no documentation."""

        @property
        def name(self) -> str:
            ...

        @name.setter
        def name(self, value: str):
            ...

        @property
        def underlying(self) -> str:
            ...

        @underlying.setter
        def underlying(self, value: str):
            ...

        @property
        def description(self) -> str:
            ...

        @description.setter
        def description(self, value: str):
            ...

        @property
        def type(self) -> str:
            ...

        @type.setter
        def type(self, value: str):
            ...

        @property
        def expiry(self) -> typing.Optional[datetime.datetime]:
            ...

        @expiry.setter
        def expiry(self, value: typing.Optional[datetime.datetime]):
            ...

        @property
        def perpetual(self) -> bool:
            ...

        @perpetual.setter
        def perpetual(self, value: bool):
            ...

        @property
        def expired(self) -> bool:
            ...

        @expired.setter
        def expired(self, value: bool):
            ...

        @property
        def enabled(self) -> bool:
            ...

        @enabled.setter
        def enabled(self, value: bool):
            ...

        @property
        def postOnly(self) -> bool:
            ...

        @postOnly.setter
        def postOnly(self, value: bool):
            ...

        @property
        def imfFactor(self) -> float:
            ...

        @imfFactor.setter
        def imfFactor(self, value: float):
            ...

        @property
        def underlyingDescription(self) -> str:
            ...

        @underlyingDescription.setter
        def underlyingDescription(self, value: str):
            ...

        @property
        def expiryDescription(self) -> str:
            ...

        @expiryDescription.setter
        def expiryDescription(self, value: str):
            ...

        @property
        def moveStart(self) -> typing.Optional[datetime.datetime]:
            ...

        @moveStart.setter
        def moveStart(self, value: typing.Optional[datetime.datetime]):
            ...

        @property
        def positionLimitWeight(self) -> float:
            ...

        @positionLimitWeight.setter
        def positionLimitWeight(self, value: float):
            ...

        @property
        def group(self) -> str:
            ...

        @group.setter
        def group(self, value: str):
            ...

    SpotType: str = "spot"

    FutureType: str = "future"

    @property
    def name(self) -> str:
        ...

    @name.setter
    def name(self, value: str):
        ...

    @property
    def enabled(self) -> bool:
        ...

    @enabled.setter
    def enabled(self, value: bool):
        ...

    @property
    def postOnly(self) -> bool:
        ...

    @postOnly.setter
    def postOnly(self, value: bool):
        ...

    @property
    def priceIncrement(self) -> float:
        ...

    @priceIncrement.setter
    def priceIncrement(self, value: float):
        ...

    @property
    def sizeIncrement(self) -> float:
        ...

    @sizeIncrement.setter
    def sizeIncrement(self, value: float):
        ...

    @property
    def type(self) -> str:
        ...

    @type.setter
    def type(self, value: str):
        ...

    @property
    def baseCurrency(self) -> str:
        ...

    @baseCurrency.setter
    def baseCurrency(self, value: str):
        ...

    @property
    def quoteCurrency(self) -> str:
        ...

    @quoteCurrency.setter
    def quoteCurrency(self, value: str):
        ...

    @property
    def restricted(self) -> bool:
        ...

    @restricted.setter
    def restricted(self, value: bool):
        ...

    @property
    def underlying(self) -> str:
        ...

    @underlying.setter
    def underlying(self, value: str):
        ...

    @property
    def highLeverageFeeExempt(self) -> bool:
        ...

    @highLeverageFeeExempt.setter
    def highLeverageFeeExempt(self, value: bool):
        ...

    @property
    def future(self) -> FtxApi.WebSocket.Models.MarketState.FutureState:
        ...

    @future.setter
    def future(self, value: FtxApi.WebSocket.Models.MarketState.FutureState):
        ...

    @property
    def id(self) -> str:
        ...

    @id.setter
    def id(self, value: str):
        ...





from typing import overload
import QuantConnect.Optimizer
import System


class OptimizationStatus(System.Enum):
    """The different optimization status"""

    New = 0
    """Just created and not running optimization"""

    Aborted = 1
    """We failed or we were aborted"""

    Running = 2
    """We are running"""

    Completed = 3
    """Optimization job has completed"""





from typing import overload
import typing

import FtxApi
import FtxApi.Rest.Models
import FtxApi.WebSocket
import FtxApi.WebSocket.Models
import System
import System.Collections.Generic
import System.Threading
import System.Threading.Tasks


class FtxSenderClientWebSocket(System.Object):
    """This class has no documentation."""

    @staticmethod
    def FtxAuthentication(webSocket: typing.Any, client: FtxApi.Client) -> System.Threading.Tasks.Task:
        ...

    @staticmethod
    def SendFtxPing(webSocket: typing.Any) -> System.Threading.Tasks.Task:
        ...

    @staticmethod
    @overload
    def SubscribeFtxChannel(webSocket: typing.Any, channel: str, market: str) -> System.Threading.Tasks.Task:
        ...

    @staticmethod
    @overload
    def SubscribeFtxChannel(webSocket: typing.Any, channel: str) -> System.Threading.Tasks.Task:
        ...

    @staticmethod
    @overload
    def UnsubscribeFtxChannel(webSocket: typing.Any, channel: str) -> System.Threading.Tasks.Task:
        ...

    @staticmethod
    @overload
    def UnsubscribeFtxChannel(webSocket: typing.Any, channel: str, market: str) -> System.Threading.Tasks.Task:
        ...


class FtxWsMarkets(System.Object, System.IDisposable):
    """This class has no documentation."""

    Url: str

    @property
    def ReceiveUpdates(self) -> typing.Callable[[System.Collections.Generic.List[FtxApi.WebSocket.Models.MarketState]], System.Threading.Tasks.Task]:
        ...

    @ReceiveUpdates.setter
    def ReceiveUpdates(self, value: typing.Callable[[System.Collections.Generic.List[FtxApi.WebSocket.Models.MarketState]], System.Threading.Tasks.Task]):
        ...

    def __init__(self, logger: typing.Any) -> None:
        ...

    def Dispose(self) -> None:
        ...

    def Start(self, ct: typing.Optional[System.Threading.CancellationToken] = None) -> None:
        ...

    def Stop(self) -> None:
        ...


class FtxWsOrderBooks(System.Object, System.IDisposable):
    """This class has no documentation."""

    Url: str

    ChannelName: str = "orderbook"

    @property
    def ReceiveUpdates(self) -> typing.Callable[[FtxApi.WebSocket.Models.FtxOrderBook], System.Threading.Tasks.Task]:
        ...

    @ReceiveUpdates.setter
    def ReceiveUpdates(self, value: typing.Callable[[FtxApi.WebSocket.Models.FtxOrderBook], System.Threading.Tasks.Task]):
        ...

    def __init__(self, logger: typing.Any) -> None:
        ...

    def Dispose(self) -> None:
        ...

    def Resubscribe(self, symbols: System.Collections.Generic.IList[str]) -> System.Threading.Tasks.Task:
        ...

    def Start(self, ct: typing.Optional[System.Threading.CancellationToken] = None) -> None:
        ...

    def Stop(self) -> None:
        ...

    def Subscribe(self, symbols: System.Collections.Generic.IList[str]) -> System.Threading.Tasks.Task:
        ...

    def Unsubscribe(self, symbols: System.Collections.Generic.IList[str]) -> System.Threading.Tasks.Task:
        ...


class FtxWsOrders(System.Object, System.IDisposable):
    """This class has no documentation."""

    Url: str

    @property
    def OrderUpdates(self) -> typing.Callable[[FtxApi.Rest.Models.Order], System.Threading.Tasks.Task]:
        ...

    @OrderUpdates.setter
    def OrderUpdates(self, value: typing.Callable[[FtxApi.Rest.Models.Order], System.Threading.Tasks.Task]):
        ...

    @property
    def FillUpdates(self) -> typing.Callable[[FtxApi.Rest.Models.Fill], System.Threading.Tasks.Task]:
        ...

    @FillUpdates.setter
    def FillUpdates(self, value: typing.Callable[[FtxApi.Rest.Models.Fill], System.Threading.Tasks.Task]):
        ...

    def __init__(self, client: FtxApi.Client, logger: typing.Any) -> None:
        ...

    def Dispose(self) -> None:
        ...

    def Start(self, ct: typing.Optional[System.Threading.CancellationToken] = None) -> None:
        ...

    def Stop(self) -> None:
        ...


class FtxWsPrices(System.Object, System.IDisposable):
    """This class has no documentation."""

    Url: str

    ChannelName: str = "ticker"

    @property
    def ReceiveUpdates(self) -> typing.Callable[[FtxApi.WebSocket.Models.FtxTicker], System.Threading.Tasks.Task]:
        ...

    @ReceiveUpdates.setter
    def ReceiveUpdates(self, value: typing.Callable[[FtxApi.WebSocket.Models.FtxTicker], System.Threading.Tasks.Task]):
        ...

    def __init__(self, logger: typing.Any) -> None:
        ...

    def Dispose(self) -> None:
        ...

    def Resubscribe(self, symbols: System.Collections.Generic.IList[str]) -> System.Threading.Tasks.Task:
        ...

    def Start(self, ct: typing.Optional[System.Threading.CancellationToken] = None) -> None:
        ...

    def Stop(self) -> None:
        ...

    def Subscribe(self, symbols: System.Collections.Generic.IList[str]) -> System.Threading.Tasks.Task:
        ...

    def Unsubscribe(self, symbols: System.Collections.Generic.IList[str]) -> System.Threading.Tasks.Task:
        ...





from typing import overload
import typing

import FtxApi.WsEngine
import System
import System.Threading.Tasks

ClientWebSocket = typing.Any


class WebsocketEngine(System.Object, System.IDisposable):
    """This class has no documentation."""

    InitBufferSize: int

    @property
    def OnConnect(self) -> typing.Callable[[ClientWebSocket], System.Threading.Tasks.Task]:
        ...

    @OnConnect.setter
    def OnConnect(self, value: typing.Callable[[ClientWebSocket], System.Threading.Tasks.Task]):
        ...

    @property
    def OnDisconnect(self) -> typing.Callable[[], System.Threading.Tasks.Task]:
        ...

    @OnDisconnect.setter
    def OnDisconnect(self, value: typing.Callable[[], System.Threading.Tasks.Task]):
        ...

    @property
    def OnReceive(self) -> typing.Callable[[ClientWebSocket, str], System.Threading.Tasks.Task]:
        ...

    @OnReceive.setter
    def OnReceive(self, value: typing.Callable[[ClientWebSocket, str], System.Threading.Tasks.Task]):
        ...

    @property
    def SendPing(self) -> typing.Callable[[ClientWebSocket], System.Threading.Tasks.Task]:
        ...

    @SendPing.setter
    def SendPing(self, value: typing.Callable[[ClientWebSocket], System.Threading.Tasks.Task]):
        ...

    def __init__(self, name: str, url: str, pingIntervalMSec: int, silenceDisconnectIntervalMSec: int, logger: typing.Any) -> None:
        ...

    def Dispose(self) -> None:
        ...

    def GetClientWebSocket(self) -> typing.Optional[ClientWebSocket]:
        ...

    def OnConnectAsync(self, clientWebSocket: typing.Any) -> System.Threading.Tasks.Task:
        ...

    def OnDisconnectAsync(self) -> System.Threading.Tasks.Task:
        ...

    def OnReceiveAsync(self, webSocket: typing.Any, message: str) -> System.Threading.Tasks.Task:
        ...

    def Run(self) -> System.Threading.Tasks.Task:
        ...

    def SendPingMessage(self, clientWebSocket: typing.Any) -> System.Threading.Tasks.Task:
        ...

    def Start(self) -> None:
        ...

    def Stop(self) -> None:
        ...





from typing import overload
import Calculators
import System


class SecurityType(System.Enum):
    """This class has no documentation."""

    # Cannot convert to Python: None = 0

    Spot = 1

    Futures = 2





from typing import overload
import abc
import typing

import Microsoft.CodeAnalysis
import Microsoft.CodeAnalysis.CSharp
import Microsoft.CodeAnalysis.CSharp.Syntax
import QuantConnectStubsGenerator.Model
import QuantConnectStubsGenerator.Parser
import System
import System.Xml


class TypeConverter(System.Object):
    """The TypeConverter is responsible for converting AST nodes into PythonType instances."""

    def __init__(self, model: Microsoft.CodeAnalysis.SemanticModel) -> None:
        ...

    def GetSymbol(self, node: Microsoft.CodeAnalysis.SyntaxNode) -> Microsoft.CodeAnalysis.ISymbol:
        """
        Returns the symbol of the given node.
        Returns null if the semantic model does not contain a symbol for the node.
        """
        ...

    @overload
    def GetType(self, node: Microsoft.CodeAnalysis.SyntaxNode, skipPythonTypeCheck: bool = False) -> QuantConnectStubsGenerator.Model.PythonType:
        """
        Returns the Python type of the given node.
        Returns an aliased typing.Any if there is no Python type for the given symbol.
        """
        ...

    @overload
    def GetType(self, symbol: Microsoft.CodeAnalysis.ISymbol, skipPythonTypeCheck: bool = False) -> QuantConnectStubsGenerator.Model.PythonType:
        """
        Returns the Python type of the given symbol.
        Returns an aliased typing.Any if there is no Python type for the given symbol.
        """
        ...


class BaseParser(Microsoft.CodeAnalysis.CSharp.CSharpSyntaxWalker, metaclass=abc.ABCMeta):
    """This class has no documentation."""

    @property
    def _context(self) -> QuantConnectStubsGenerator.Model.ParseContext:
        """This field is protected."""
        ...

    @property
    def _model(self) -> Microsoft.CodeAnalysis.SemanticModel:
        """This field is protected."""
        ...

    @property
    def _typeConverter(self) -> QuantConnectStubsGenerator.Parser.TypeConverter:
        """This field is protected."""
        ...

    @property
    def _currentNamespace(self) -> QuantConnectStubsGenerator.Model.Namespace:
        """This field is protected."""
        ...

    @_currentNamespace.setter
    def _currentNamespace(self, value: QuantConnectStubsGenerator.Model.Namespace):
        """This field is protected."""
        ...

    @property
    def _currentClass(self) -> QuantConnectStubsGenerator.Model.Class:
        """This field is protected."""
        ...

    @_currentClass.setter
    def _currentClass(self, value: QuantConnectStubsGenerator.Model.Class):
        """This field is protected."""
        ...

    def __init__(self, context: QuantConnectStubsGenerator.Model.ParseContext, model: Microsoft.CodeAnalysis.SemanticModel) -> None:
        """This method is protected."""
        ...

    def AppendSummary(self, currentSummary: str, text: str) -> str:
        """
        Appends the given text to the given summary.
        An empty line is placed between the current summary and the given text.
        
        This method is protected.
        """
        ...

    def EnterClass(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.BaseTypeDeclarationSyntax) -> None:
        """
        EnterClass is the method that is called whenever a class/struct/enum/interface is entered.
        In the BaseParser it is assumed that the class that is entered is already registered in the namespace.
        In the ClassParser, which runs before any other parsers, this method is overridden to register classes.
        
        This method is protected.
        """
        ...

    def FormatValue(self, value: str) -> str:
        """
        Format a default C# value into a default Python value.
        
        This method is protected.
        """
        ...

    def GetDeprecationReason(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.MemberDeclarationSyntax) -> str:
        """
        Returns the deprecation message if the node is marked obsolete, or null if it is not.
        
        This method is protected.
        """
        ...

    def HasModifier(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.MemberDeclarationSyntax, modifier: str) -> bool:
        """
        Check if a node has a modifier like private or static.
        
        This method is protected.
        """
        ...

    def ParseDocumentation(self, node: Microsoft.CodeAnalysis.SyntaxNode) -> System.Xml.XmlElement:
        """
        Parses the documentation above a node to an XML element.
        If the documentation contains a summary, this is then accessible with element["summary"].
        
        This method is protected.
        """
        ...

    def VisitClassDeclaration(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.ClassDeclarationSyntax) -> None:
        ...

    def VisitEnumDeclaration(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.EnumDeclarationSyntax) -> None:
        ...

    def VisitInterfaceDeclaration(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.InterfaceDeclarationSyntax) -> None:
        ...

    def VisitNamespaceDeclaration(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.NamespaceDeclarationSyntax) -> None:
        ...

    def VisitStructDeclaration(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.StructDeclarationSyntax) -> None:
        ...


class ClassParser(QuantConnectStubsGenerator.Parser.BaseParser):
    """This class has no documentation."""

    def __init__(self, context: QuantConnectStubsGenerator.Model.ParseContext, model: Microsoft.CodeAnalysis.SemanticModel) -> None:
        ...

    def EnterClass(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.BaseTypeDeclarationSyntax) -> None:
        """This method is protected."""
        ...


class MethodParser(QuantConnectStubsGenerator.Parser.BaseParser):
    """This class has no documentation."""

    def __init__(self, context: QuantConnectStubsGenerator.Model.ParseContext, model: Microsoft.CodeAnalysis.SemanticModel) -> None:
        ...

    def VisitConstructorDeclaration(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.ConstructorDeclarationSyntax) -> None:
        ...

    def VisitDelegateDeclaration(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.DelegateDeclarationSyntax) -> None:
        ...

    def VisitIndexerDeclaration(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.IndexerDeclarationSyntax) -> None:
        ...

    def VisitMethodDeclaration(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.MethodDeclarationSyntax) -> None:
        ...


class PropertyParser(QuantConnectStubsGenerator.Parser.BaseParser):
    """This class has no documentation."""

    def __init__(self, context: QuantConnectStubsGenerator.Model.ParseContext, model: Microsoft.CodeAnalysis.SemanticModel) -> None:
        ...

    def VisitEnumMemberDeclaration(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.EnumMemberDeclarationSyntax) -> None:
        ...

    def VisitEventDeclaration(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.EventDeclarationSyntax) -> None:
        ...

    def VisitEventFieldDeclaration(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.EventFieldDeclarationSyntax) -> None:
        ...

    def VisitFieldDeclaration(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.FieldDeclarationSyntax) -> None:
        ...

    def VisitPropertyDeclaration(self, node: Microsoft.CodeAnalysis.CSharp.Syntax.PropertyDeclarationSyntax) -> None:
        ...





from typing import overload
import typing

import FtxApi
import System

StringEnumConverter = typing.Any


class Client(System.Object):
    """This class has no documentation."""

    @property
    def ApiKey(self) -> str:
        ...

    @property
    def ApiSecret(self) -> str:
        ...

    @property
    def SubAccount(self) -> str:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, apiKey: str, apiSecret: str, subAccount: str = None) -> None:
        ...


class FtxStringEnumConverter(StringEnumConverter):
    """This class has no documentation."""

    def WriteJson(self, writer: typing.Any, value: typing.Any, serializer: typing.Any) -> None:
        ...


